package mosaic.sim.contBlocks;

import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.continuous.*;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

/**
*
*/

public class Integration  extends AbstractModelComponent{

   protected DoubleVariable output;
   protected DoubleVariable input;

   protected JLabel la;

   public Integration() {
      super();
      initPaint();
      } // BasicConnection;

   public Integration(String name) {
      super(name);
      initPaint();
      } // BasicConnection


   public Integration(String name, DoubleVariable input, DoubleVariable output){
     super(name);
     setInput(input);
     setOutput(output);
     initPaint();
   }

  protected void initPaint(){
    la=new JLabel();
    la.setFont(new java.awt.Font("Dialog", 1, 10));
    la.setForeground(Color.black);
    la.setOpaque(false);
  }

   /** connects two Variables */
   protected final void connect(){
     if (!enabled) return; // do nothing
     output.setDerivative(input);
     output.setTypeState();
   }


   /** disconnects two Variables */
   protected final void disconnect(){
     output.setDerivative(null);
     output.setTypeVariable();
   }

   /**
   * Set the input variable
   */
   public final void setInput (DoubleVariable input){
     if (this.input==input) return; // nothing changed
     this.input=input; // set new input
     if (enabled && input!=null && output!=null)
         connect(); // create new connection
     fireModelComponentChanged();; // inform listeners
   }

   /**
   * Set the output variable
   */
   public final void setOutput (DoubleVariable output){
     if (this.output==output) return; // nothing changed
     this.output=output; // set new output
     if (enabled && output!=null && input!=null)
       connect(); // create new connection
     fireModelComponentChanged();; // inform listeners
   }


   /**
   * Get the input variable
   */
   public final DoubleVariable getInput () {
      return (input);
   }

   /**
   * Get the input variable
   */
   public final DoubleVariable getOutput () {
      return (output);
   }

   public void setEnabled(boolean enabled){
     super.setEnabled(enabled); // supercall to BasicModel
     if (enabled) connect();
     else disconnect(); // remove connection
   }

   protected void finalize(){
     try{
       super.finalize();
     }catch(Throwable t){}
     disable();
   }

    /**
    * Paints the connection.
    * Paints an arrow and labels it with name of input and output.
    * @param g      the graphics context of the component
    */
  public void paintComponent(Graphics g) {
    Graphics2D g2=(Graphics2D) g;
    int h = getSize().height-1;
    int w = getSize().width-1;
    int minWH=Math.min(h,w/2);

    Color lineC;
    Color fillC;
/*    if (enabled) g.setColor(Color.black);
    else g.setColor(Color.gray);*/
    if (enabled){
      lineC=Color.black;
      fillC=Color.yellow;
    }
    else{
      lineC=Color.gray;
      fillC=Color.lightGray;
    }
    g2.setPaint(fillC);

//    Rectangle r=new Rectangle(w/4,0,w/2,h);
    Ellipse2D e=new Ellipse2D.Double(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    g2.fill(e);

    g2.setPaint(lineC);
    g2.draw(e);
    g2.draw(new Line2D.Double(0,h/2,w/2-minWH/2,h/2));
    g2.draw(new Line2D.Double(w/2+minWH/2,h/2,w,h/2));

//    la.setBounds((x1+x2)/2+5,(y1+y2)/2-10,(int)la.getPreferredSize().getWidth(),(int)la.getPreferredSize().getHeight());
    la.setHorizontalAlignment(la.CENTER);
    la.setVerticalAlignment(la.CENTER);

    la.setBounds(e.getBounds());
    Graphics2D g3=(Graphics2D)g2.create(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    la.setText("INT");
    la.paint(g3);

    if (input!=null){
      la.setBounds(0,0,w/2-minWH/2,h/2);
      la.setText(input.getLabel());
      la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.setHorizontalAlignment(la.LEFT);
      g3=(Graphics2D)g2.create(0,0,w/2-minWH/2,h/2);
      la.paint(g3);
    }
    if (output!=null){
      la.setBounds(w/2+minWH/2,0,w/2-minWH/2,h/2);
      la.setText(output.getLabel());
      la.setHorizontalAlignment(la.RIGHT);
      g3=(Graphics2D)g2.create(w/2+minWH/2,0,w/2-minWH/2,h/2);
      if (input==null){
        la.setFont(new java.awt.Font("Dialog", 0, 10));
        la.setVerticalAlignment(la.BOTTOM);
      }
      la.paint(g3);
    }

  }

  public Dimension getPreferredSize() {
    return new Dimension(58,24);
  }

}
// end of BasicConnection.java