package mosaic.sim.contBlocks;

import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.JLabel;
import java.util.ArrayList;


/** Implements the subtraction. */
public class Subtraction extends AbstractContEquation{
   DoubleVariable subtrahend;DoubleVariable minuend;DoubleVariable output;
   JLabel la;
   public Subtraction() {
      super();
      initPaint();
      } // Subtraction;

   public Subtraction(String name) {
      super(name);
      initPaint();
      } // Subtraction

   public Subtraction(DoubleVariable minuend,DoubleVariable subtrahend,DoubleVariable output){
      super();
      initPaint();
      setMinuend(minuend);
      setSubtrahend(subtrahend);
      setOutput(output);
      } // Subtraction;

   public Subtraction(String name,DoubleVariable minuend,DoubleVariable subtrahend,DoubleVariable output){
      super(name);
      initPaint();
      setMinuend(minuend);
      setSubtrahend(subtrahend);
      setOutput(output);
      } // Subtraction


  protected void initPaint(){
    la=new JLabel();
    la.setFont(new java.awt.Font("Dialog", 1, 10));
    la.setForeground(Color.black);
    la.setOpaque(false);
  }

  /** Set the minuend. */
  public void setMinuend(DoubleVariable minuend){
    try{
      this.minuend=minuend;
      ArrayList l=new ArrayList(2);
      if (subtrahend!=null) l.add(subtrahend);
      if (minuend!=null) l.add(minuend);
      DoubleVariable domains[]=new DoubleVariable[l.size()];
      for (int i=0;i<l.size();i++)domains[i]=(DoubleVariable)l.get(i);
      setDomains(domains);
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Set the divisor. */
  public void setSubtrahend(DoubleVariable subtrahend){
    try{
      this.subtrahend=subtrahend;
      ArrayList l=new ArrayList(2);
      if (subtrahend!=null) l.add(subtrahend);
      if (minuend!=null) l.add(minuend);
      DoubleVariable domains[]=new DoubleVariable[l.size()];
      for (int i=0;i<l.size();i++)domains[i]=(DoubleVariable)l.get(i);
      setDomains(domains);
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }


  /** Set the output variable. */
  public void setOutput(DoubleVariable output){
    this.output=output;
    try{
      setTargets(new DoubleVariable[]{output});
    } catch(VariableException e){printErrorOutput(e.getMessage());}
  }

  /** Returns the divident. */
  public DoubleVariable getMinuend(){
    return minuend;
  }

  /** Returns the divisor. */
  public DoubleVariable getSubtrahend(){
    return subtrahend;
  }

  /** Returns the output variable. */
  public DoubleVariable getOutput(){
    return output;
  }

  public void equation(){
    output.setValue(minuend.getValue()-subtrahend.getValue());
  }

  public void paintComponent(Graphics g) {
    Graphics2D g2=(Graphics2D) g;
    int h = getSize().height-1;
    int w = getSize().width-1;
    int minWH=Math.min(h,w/2);
    int dist=Math.max(h/10,2);

    Color lineC;
    Color fillC;
/*    if (enabled) g.setColor(Color.black);
    else g.setColor(Color.gray);*/
    if (enabled){
      lineC=Color.black;
      fillC=Color.yellow;
    }
    else{
      lineC=Color.gray;
      fillC=Color.lightGray;
    }
    g2.setPaint(fillC);

//    Rectangle r=new Rectangle(w/4,0,w/2,h);
    Ellipse2D e=new Ellipse2D.Double(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    g2.fill(e);

    g2.setPaint(lineC);
    g2.draw(e);
    g2.draw(new Line2D.Double(0,h/2+dist,w/2-minWH/2,h/2+dist));
    g2.draw(new Line2D.Double(0,h/2-dist,w/2-minWH/2,h/2-dist));
    g2.draw(new Line2D.Double(w/2+minWH/2,h/2,w,h/2));

//    la.setBounds((x1+x2)/2+5,(y1+y2)/2-10,(int)la.getPreferredSize().getWidth(),(int)la.getPreferredSize().getHeight());
    la.setHorizontalAlignment(la.CENTER);
    la.setVerticalAlignment(la.CENTER);

    la.setBounds(e.getBounds());
    Graphics2D g3=(Graphics2D)g2.create(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    la.setText("SUB");
    la.paint(g3);

    if (minuend!=null){
      la.setBounds(0,0,w/2-minWH/2,h/2-dist);
      la.setText(minuend.getLabel());
      la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.setHorizontalAlignment(la.LEFT);
      g3=(Graphics2D)g2.create(0,0,w/2-minWH/2,h/2);
      la.paint(g3);
    }
    if (subtrahend!=null){
      la.setBounds(0,h/2+dist,w/2-minWH/2,h);
      la.setText(subtrahend.getLabel());
      g3=(Graphics2D)g2.create(0,h/2+dist,w/2-minWH/2,h);
      if (minuend==null){
        la.setHorizontalAlignment(la.LEFT);
        la.setFont(new java.awt.Font("Dialog", 0, 10));
      }
      la.setVerticalAlignment(la.TOP);
      la.paint(g3);
    }


    if (output!=null){
      la.setBounds(w/2+minWH/2,0,w/2-minWH/2,h/2);
      la.setText(output.getLabel());
      la.setHorizontalAlignment(la.RIGHT);
      g3=(Graphics2D)g2.create(w/2+minWH/2,0,w/2-minWH/2,h/2);
      if (minuend==null&&subtrahend==null){
        la.setFont(new java.awt.Font("Dialog", 0, 10));
      }
      la.setVerticalAlignment(la.BOTTOM);
      la.paint(g3);
    }

  }

  public Dimension getPreferredSize() {
    return new Dimension(58,24);
  }


}