/**
 * Project mosaic
 *
 * AbstractModelComponentVisitor.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import java.util.*;

/**
* Abstract implementation for interface ModelComponentVisitor.
* Use this as a base class for all visitors to perform operations on the hierarchy
* of ModelComponents.
*/
abstract public class AbstractModelComponentVisitor implements ModelComponentVisitor {

    public AbstractModelComponentVisitor() {
    }

    public AbstractModelComponentVisitor(ModelComponent root) {
        visit(root);
    }

    /**
    * Visit the ModelComponent and all subcomponents in the hierarchy and perform an operation
    * (call performOperation for the visited ModelComponent).
    * @param c the ModelComponent visited.
    */
    public void visit(ModelComponent c){
  	    if (c instanceof ModelComponent) {
  	        Iterator subCompElems = ((ModelComponent)c).getModelComponents().iterator();
  	        while (subCompElems.hasNext()) {
  	            visit((ModelComponent)subCompElems.next());
  	        }
  	    }
  	    performOperation (c);
    }

    /** do an operation on the visited ModelComponent */
    abstract public void performOperation (ModelComponent c);

}

