/**
 * Project mosaic
 *
 * SimOptions.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.continuous.*;

/*
 * Class to hold the global options for a simulation application.
 * Is used by the options dialog to set the options.
 * Then sets the fields in various simulation objects, like scheduler, itegrator, ...
 */
public class SimOptions {

    /** Default constructor without arguments. */
    public SimOptions(SimApplication appl) {
        setSimApplication(appl);
    }

// simApplication ----------------------------------------------------------------------------------

    /** The simulation application object this option belongs to */
    protected SimApplication simAppl;

    /** Gets the simulation application object this option belongs to */
    protected SimApplication getSimApplication() {
        return simAppl;
    }

    /** Sets the simulation application object this option belongs to */
    protected void setSimApplication(SimApplication appl) {
        simAppl = appl;
    }



// scheduling strategy -----------------------------------------------------------------------------

    /** Sets the scheduling strategy as a text string */
    public void setStrategyAsText(String newStrategy) {
        getSimApplication().getSimulationScheduler().setStrategyAsText(newStrategy);
    }

    /** Gets the scheduling strategy as a text string */
    public String getStrategyAsText() {
        return getSimApplication().getSimulationScheduler().getStrategyAsText();
    }

    /** gets the scheduling strategy object */
    public SchedulingStrategy getSchedulingStrategy() {
        return getSimApplication().getSimulationScheduler().getSchedulingStrategy();
    }


// time unit of simulation time --------------------------------------------------------------------

    /** Sets the time unit for the simulation time. Used in real time mode. */
    public void setTimeUnitAsText(String timeUnit) {
        getSimApplication().getSimulationScheduler().setTimeUnitAsText(timeUnit);
    }

    /** Gets the time unit for the simulation time. Used in real time mode. */
    public String getTimeUnitAsText() {
        return getSimApplication().getSimulationScheduler().getTimeUnitAsText();
    }

//animation rate -----------------------------------------------------------------------------------

    /** Gets the time delay between events. */
    public long getAnimationRate() {
        return getSimApplication().getSimulationScheduler().getAnimationRate();
    }

    /** Sets the time delay between events. */
    public void setAnimationRate(long newAnimationRate) {
        getSimApplication().getSimulationScheduler().setAnimationRate(newAnimationRate);
    }

// integration method ------------------------------------------------------------------------------

   /** Gets the integration method as a String */
   public final String getIntegrationMethodAsText() {
      return getSimApplication().getIntegrator().getIntegrationMethodAsText();
   }

   /** Sets the integration method from a String */
   public final void setIntegrationMethodAsText(String integrationMethodName) {
      getSimApplication().getIntegrator().setIntegrationMethodAsText(integrationMethodName);
   }

   /** Gets the integration method as a String */
   public final IntegrationMethod getIntegrationMethod() {
      return getSimApplication().getIntegrator().getIntegrationMethod();
   }


// step width --------------------------------------------------------------------------------------

   /**
   * Returns the stepwidth for the integration steps
   */
   public final double getStepWidth()  {
      return getSimApplication().getIntegrator().getStepWidth();
   }

   /**
   * Sets the stepwidth for the next integration step.
   */
   public final void setStepWidth(double stepWidth) {
       getSimApplication().getIntegrator().setStepWidth(stepWidth);
   }


// epsilon tolerance for state event detection -----------------------------------------------------

   /**
   * Returns the epsilon value for state event detection.
   */
   public final double getStateEventTolerance() {
      return getSimApplication().getIntegrator().getStateEventTolerance();
   }

   /**
   * Sets the  epsilon value for state event detection.
   */
   public final void setStateEventTolerance(double tolerance) {
      getSimApplication().getIntegrator().setStateEventTolerance(tolerance);
   }

// sample rate for sampling continuous values ------------------------------------------------------

   /**
   * Returns the rate for the output of continuous values
   */
   public final int getSampleRate()  {
      return getSimApplication().getIntegrator().getSampleRate();
   }

   /**
   * Sets the sample rate  for the next integration step.
   */
   public final void setSampleRate(int sampleRate) {
       getSimApplication().getIntegrator().setSampleRate(sampleRate);
   }


// path to image files.... -------------------------------------------------------------------------

    /** path to image files */
    public String imagesPath = "/sim/img/";

}
