/**
 * Project mosaic
 *
 * SimOptionsDlg.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import mosaic.util.custom.RTPropertyDialog;
import mosaic.sim.kernel.control.*;

// =================================================================================================
/*
 * The SimOptionsDlg provides options for the simulation such as:
 * - Time unit    (ms, s, m, h, d, ...)
 * - Simulation Mode (Realtime, ...)
 * - Continuous options (StepWidth, Numerical Integr. Method)
 */
public class SimOptionsDlg extends JDialog implements ActionListener {

    public static boolean[] DEBUG = new boolean[] { true };
    protected SimMainFrame parentSimFrame;

// GUI components ----------------------------------------------------------------------------------

    JPanel optionPanel = new JPanel();
    //ok panel
    JButton okButton = new JButton();
    JButton cancelButton = new JButton();
    JPanel okPanel = new JPanel();
    BorderLayout okLayout = new BorderLayout();
    //simPanel
    JPanel simPanel = new JPanel();
    TitledBorder simBorder;
    JButton modeButton = new JButton();
    JLabel modeLabel = new JLabel();
    JComboBox modeCombo = new JComboBox();
    JTextField stepWidthField = new JTextField();
    //cont
    TitledBorder contBorder;
    JPanel contPanel = new JPanel();
    JLabel timeUnitLabel = new JLabel();
    JComboBox timeUnitCombo = new JComboBox();
    JLabel imLabel1 = new JLabel();
    JLabel imLabel2 = new JLabel();
    JComboBox imCombo = new JComboBox();
    JLabel stepWidthLabel = new JLabel();
    JTextField toleranceField = new JTextField();
    JLabel imLabel4 = new JLabel();
    JLabel toleranceLabel2 = new JLabel();
    JLabel imLabel6 = new JLabel();
    JTextField sampleField = new JTextField();
    JLabel modelVisibilityLabel1 = new JLabel();
    JComboBox modelsVisibleCombo = new JComboBox();

// Construction ------------------------------------------------------------------------------------

    public SimOptionsDlg(SimMainFrame owner, String title, boolean modal) {
        super(owner, title, modal);
        parentSimFrame = owner;
        try {
            jbInit();
        } catch (Exception e) {
            owner.getSimApplication().errorOutput("SimOptionsDlg exception " + e);
        }
        getContentPane().add(optionPanel);
        init();
        pack();
        setSize(265, 420);
        this.setResizable(false);
    }

    public void update() {
        updateGUI();
    }

    private void jbInit() throws Exception {
        contBorder = new TitledBorder("");
        simBorder = new TitledBorder("");

        optionPanel.setLayout(null);

        // OkPanel
        okPanel.setBounds(new Rectangle(8, 356, 248, 32));
        okButton.setPreferredSize(new Dimension(73, 21));
        okButton.setText("OK");
        cancelButton.setText("Cancel");
        okPanel.setLayout(okLayout);
        contPanel.setPreferredSize(new Dimension(20, 20));
        contPanel.setBounds(new Rectangle(38, 31, 1, 1));
        toleranceField.setBounds(new Rectangle(97, 63, 117, 24));
        toleranceField.setText("0.01");
        imLabel4.setFont(new java.awt.Font("Dialog", 1, 11));
        imLabel4.setText("tolerance :");
        toleranceLabel2.setFont(new java.awt.Font("Dialog", 1, 11));
        toleranceLabel2.setText("State event");
        toleranceLabel2.setBounds(new Rectangle(10, 86, 82, 32));
        imLabel6.setBounds(new Rectangle(9, 58, 82, 32));
        imLabel6.setFont(new java.awt.Font("Dialog", 1, 11));
        imLabel6.setText("Output rate:");
        sampleField.setBounds(new Rectangle(101, 119, 123, 24));
        sampleField.setText("10");
        okLayout.setHgap(3);
        okLayout.setVgap(3);

        // ModelPanel
        modelVisibilityLabel1.setFont(new java.awt.Font("Dialog", 1, 11));
        modelVisibilityLabel1.setText("Models visible :");
        modelVisibilityLabel1.setBounds(new Rectangle(8, 23, 89, 31));
        modelsVisibleCombo.setBounds(new Rectangle(100, 27, 123, 28));
        timeUnitLabel.setFont(new java.awt.Font("Dialog", 1, 11));
        modeLabel.setFont(new java.awt.Font("Dialog", 1, 11));
        imLabel1.setFont(new java.awt.Font("Dialog", 1, 11));
        stepWidthLabel.setFont(new java.awt.Font("Dialog", 1, 11));
        imLabel2.setFont(new java.awt.Font("Dialog", 1, 11));
        optionPanel.add(simPanel, null);
        simPanel.add(modeButton, null);
        simPanel.add(modelVisibilityLabel1, null);
        simPanel.add(timeUnitLabel, null);
        simPanel.add(modeLabel, null);
        simPanel.add(modeCombo, null);
        simPanel.add(modelsVisibleCombo, null);
        simPanel.add(timeUnitCombo, null);
        optionPanel.add(okPanel, null);
        okPanel.add(okButton, BorderLayout.WEST);
        okPanel.add(cancelButton, BorderLayout.EAST);
        optionPanel.add(contPanel, null);
        contPanel.add(imLabel1, null);
        contPanel.add(imLabel2, null);
        contPanel.add(stepWidthLabel, null);
        contPanel.add(imLabel6, null);
        contPanel.add(imLabel4, null);
        contPanel.add(toleranceLabel2, null);
        contPanel.add(stepWidthField, null);
        contPanel.add(imCombo, null);
        contPanel.add(toleranceField, null);
        contPanel.add(sampleField, null);

        modeLabel.setBounds(new Rectangle(8, 133, 69, 31));
        modeLabel.setText("Mode :");
        modeCombo.setBounds(new Rectangle(99, 135, 123, 28));
        modeButton.setText("...");
        modeButton.setBounds(new Rectangle(223, 135, 21, 28));

        simPanel.setBounds(new Rectangle(8, 12, 250, 180));
        simPanel.setBorder(simBorder);
        simPanel.setLayout(null);
        simBorder.setTitle("Simulation");

        // ContPanel
        timeUnitLabel.setText("Time unit :");
        timeUnitLabel.setBounds(new Rectangle(9, 97, 69, 31));
        timeUnitCombo.setBounds(new Rectangle(101, 100, 123, 28));
        imLabel1.setText("Integration");
        imLabel1.setBounds(new Rectangle(10, 26, 82, 18));
        imLabel2.setText("method :");
        imCombo.setBounds(new Rectangle(102, 26, 123, 28));
        stepWidthLabel.setText("Stepwidth :");
        imLabel2.setBounds(new Rectangle(10, 36, 82, 24));
        stepWidthLabel.setBounds(new Rectangle(10, 59, 82, 32));
        stepWidthField.setText("0.1");
        stepWidthField.setBounds(new Rectangle(101, 61, 123, 24));
        contBorder.setTitle("Continuous");

        contPanel.setBounds(new Rectangle(9, 192, 250, 154));
        contPanel.setLayout(null);
        contPanel.setBorder(contBorder);

        optionPanel.add(contPanel, null);
        optionPanel.add(simPanel, null);
        toleranceField.setBounds(new Rectangle(102, 90, 123, 24));
        imLabel4.setBounds(new Rectangle(10, 98, 82, 32));
        imLabel6.setBounds(new Rectangle(10, 116, 82, 32));
    }


// reading, gui updating and applying options ------------------------------------------------------

    private void init() {
        initGUI();
        updateGUI();
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
        modeButton.addActionListener(this);
    }

    private void initGUI() {
        modeCombo.setModel(new DefaultComboBoxModel(StaticProjectSettings.
                getSchedulingModes()));
        timeUnitCombo.setModel(new DefaultComboBoxModel(
                StaticProjectSettings.getTimeUnits()));
        imCombo.setModel(new DefaultComboBoxModel(StaticProjectSettings.
                getIntegrationMethods()));
        modelsVisibleCombo.setModel(new DefaultComboBoxModel(new String[] {"true", "false"}));
    }

    private void updateGUI() {
        String schedulingStrategy;
        String timeUnit;
        String integrationMethod;
        boolean modelsVisible;
        double tolerance;
        double stepWidth;
        int sampleRate;

        integrationMethod = parentSimFrame.getSimApplication().getOptions().getIntegrationMethodAsText();
        schedulingStrategy = parentSimFrame.getSimApplication().getOptions().getStrategyAsText();
        timeUnit = parentSimFrame.getSimApplication().getSimulationScheduler().getTimeUnitAsText();
        stepWidth = parentSimFrame.getSimApplication().getOptions().getStepWidth();
        tolerance = parentSimFrame.getSimApplication().getOptions().getStateEventTolerance();
        sampleRate = parentSimFrame.getSimApplication().getOptions().getSampleRate();

        modeCombo.setSelectedItem(schedulingStrategy);
        imCombo.setSelectedItem(integrationMethod);
        timeUnitCombo.setSelectedItem(timeUnit);
        stepWidthField.setText("" + stepWidth);
        toleranceField.setText("" + tolerance);
        sampleField.setText("" + sampleRate);

    }


    private void applyOptions() {
        String schedulingStrategy;
        String timeUnit;
        String integrationMethod;
        String modelsVisible;
        double tolerance;
        double stepWidth;
        int sampleRate;

        //simulation options
        schedulingStrategy = (String)modeCombo.getSelectedItem();
        parentSimFrame.getSimApplication().getOptions().setStrategyAsText(schedulingStrategy);
        timeUnit = (String)timeUnitCombo.getSelectedItem();
        parentSimFrame.getSimApplication().getOptions().setTimeUnitAsText(timeUnit);
        modelsVisible = (String)modelsVisibleCombo.getSelectedItem();
        // continuous options
        integrationMethod = (String)imCombo.getSelectedItem();
        stepWidth = Double.parseDouble(stepWidthField.getText());
        tolerance = Double.parseDouble(toleranceField.getText());
        sampleRate = Integer.parseInt(sampleField.getText());
        parentSimFrame.getSimApplication().getOptions().setStepWidth(stepWidth);
        parentSimFrame.getSimApplication().getOptions().setStateEventTolerance(tolerance);
        parentSimFrame.getSimApplication().getOptions().setIntegrationMethodAsText(integrationMethod);
        parentSimFrame.getSimApplication().getOptions().setSampleRate(sampleRate);
    }

// Actionlistener Interface (OK / CANCEL Buttons)  -------------------------------------------------

    public void actionPerformed(ActionEvent e) {
        SchedulingStrategy curStrat;
        RTPropertyDialog rt;

        if (e.getSource() == okButton) {
            applyOptions();
            setVisible(false);
        } else if (e.getSource() == cancelButton) {
            setVisible(false);
        } else if (e.getSource() == modeButton) {
            curStrat = parentSimFrame.getSimApplication().getOptions().getSchedulingStrategy();
            rt = new RTPropertyDialog(curStrat);
        } else {
        }
    }

} // ===============================================================================================


