/**
 * Project mosaic
 *
 * TabbedMainFrame.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;

//==================================================================================================

/**
* Class for a simulation frame which allows for multiple panels which can be changed by tabs.
*/
public class TabbedMainFrame extends SimMainFrame implements ChangeListener {

    /** currently visible panel */
    ModelPanel visiblePanel;
    /** Tabbed pane to hold the simulation panels. */
    JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);

    /** Constructor with application as argument */
    public TabbedMainFrame(SimApplication application) {
        super(application);
        try  {
            jbInit();
        } catch(Exception e) {

            e.printStackTrace(getSimApplication().getLogPrinter().getLogStream("error"));
        }
   }

    /** Private initialization method. Initializes the tabbed panes */
    private void jbInit() throws Exception {
        this.getContentPane().add(tabbedPane);
        tabbedPane.addChangeListener(this);
    }

    /** Adds a new model panel */
    public void addTab(ModelPanel simPanel) {
        JScrollPane scrollPane = new JScrollPane(simPanel);
        tabbedPane.addTab(simPanel.getLabel(), scrollPane);
        addModelPanel(simPanel);
        if (tabbedPane.getSelectedComponent() instanceof JScrollPane) {
            if ((((JScrollPane)tabbedPane.getSelectedComponent()).getViewport().getView()) instanceof ModelPanel)
                visiblePanel = (ModelPanel)(((JScrollPane)tabbedPane.getSelectedComponent()).getViewport().getView());
        }
    }

// modelPane ---------------------------------------------------------------------------------------

    /** one single model panel should contain the model */
    ModelPanel modelPanel;

    /** Sets the model panel */
    public void setModelPanel(ModelPanel panel) {
        modelPanel = panel;
        addModelPanel(panel);
        doReset();
    }

    /** Gets the model panel */
    public ModelPanel getModelPanel() {
        return modelPanel;
    }

    /** Reacts to selection events in the  tabs */
    public void stateChanged(ChangeEvent event)
    {   ModelPanel nowVisiblePanel;
        Component panel = tabbedPane.getSelectedComponent();
        if ( panel instanceof JScrollPane) {
            if ( (((JScrollPane)tabbedPane.getSelectedComponent()).getViewport().getView()) instanceof ModelPanel) {
                nowVisiblePanel = (ModelPanel)(((JScrollPane)tabbedPane.getSelectedComponent()).getViewport().getView());
                if (visiblePanel != null) {
                    if (visiblePanel != nowVisiblePanel) {
                        if (visiblePanel instanceof AttachableModelPanel)
                            ((AttachableModelPanel)visiblePanel).detach();
                        if (nowVisiblePanel instanceof AttachableModelPanel)
                            ((AttachableModelPanel)nowVisiblePanel).attach();
                        visiblePanel = nowVisiblePanel;
                    }

                }else {
                    if (nowVisiblePanel instanceof AttachableModelPanel)
                        ((AttachableModelPanel)nowVisiblePanel).attach();
                }
            }
        }
    }

} // end TabbedMainFram =============================================================================

