//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class BooleanInputView    ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.input;

import mosaic.sim.kernel.output.*;
import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.event.*;

/**
* Animation and input view to visualize and enter boolean values.
* Representation of inputview is defined in derived classes, implements
* only general behaviour for entering boolean values.
* <p>
* Use the view by adding it as VariableChangeListener to a variable of arbitrary type .
* Use the input functionality by registering a userInputListener at this.
* <p>
* Costumization:
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*/

public class BooleanInputView extends VariableInputView {

  /** Default constructor for TextInputView */
  public BooleanInputView() {
    try  {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
  }

  private void jbInit() throws Exception {
    setValue(boolValue);
 	}

  public void setBounds(int x, int y, int w, int h)
  { super.setBounds(x,y,w,h);
  }

  public void setBounds(Rectangle r)
  { this.setBounds(r.x, r.y, r.width, r.height);
  }

  public Dimension getPreferredSize()
  { return (new Dimension(50,20));
  }

  public Dimension getMinimumSize()
  { return (new Dimension(30,10));
  }

// boolean value ----------------------------------------------------------------------------------------------

  /** variable to store the boolean object */
  Boolean boolValue = new Boolean(true);

  /** Sets the boolean object received by a variableChange */
  public void setBoolValue(Boolean newVal) {
    boolValue = newVal;
  }
  /** Gets the boolean object */
  public Boolean getBoolValue() { return boolValue; }

// setValue --------------------------------------------------------------------------------------------------

  /**
  * Specialized method to set a new value;
  * also sets the booleanValue.
  */
  public void setValue (Object newVal) {
    if (newVal instanceof Boolean) {
      super.setValue(newVal);
      setBoolValue ((Boolean)newVal);
    }
    else
      System.out.println(">> tried to send non Boolean to " + toString());
  }

}
