//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class NumberInputView -------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.input;

import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* Displays the number in a textfield.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

public class SliderInputView extends BasicNumberInputView {
    JSlider jSlider = new JSlider();
    JLabel jLabel = new JLabel();
    BorderLayout borderLayout1 = new BorderLayout();

    /** Default constructor */
    public SliderInputView() {
        super();
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        jSlider.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                jSlider_stateChanged(e);
            }
        });
        jLabel.setHorizontalAlignment(SwingConstants.CENTER);
        jLabel.setText( (new Double(jSlider.getValue())).toString());
        this.add(jSlider, BorderLayout.CENTER);
        this.add(jLabel, BorderLayout.SOUTH);
    }

    void jSlider_stateChanged(ChangeEvent e) {
        jLabel.setText(new Double(((JSlider)e.getSource()).getValue()).toString());
        if ( ! ((JSlider)e.getSource()).getValueIsAdjusting() ) {
            setValue( new Double(((JSlider)e.getSource()).getValue()) );
            triggerEvent();
        }

    }

    // Overwrite paintComponent to avoid painting of background rectangle and
    // label
    public void paintComponent(Graphics g) {}

    // returns label
    public JLabel getJLabel() { return jLabel; }

    // returns slider
    public JSlider getSlider() { return jSlider; }

    // Properties forwarded to jslider
    public void setExtent(int ext)         { jSlider.setExtent(ext);           }

    public int getExtent()                 { return jSlider.getExtent();       }

    public void setMajorTickSpacing(int n) { jSlider.setMajorTickSpacing(n);   }

    public int getMajorTickSpacing()       { return jSlider.getMajorTickSpacing(); }

    public void setMaximum(int max)        { jSlider.setMaximum(max);          }

    public int getMaximum()                { return jSlider.getMaximum();      }

    public void setMinimum(int min)        { jSlider.setMinimum(min);          }

    public int getMinimum()                { return jSlider.getMinimum();      }

    public void setMinorTickSpacing(int n) { jSlider.setMinorTickSpacing(n);   }

    public int getMinorTickSpacing()       { return jSlider.getMinorTickSpacing(); }

    public void setOrientation(int o)      { jSlider.setOrientation(o);        }

    public int getOrientation()            { return jSlider.getOrientation();  }

    public void setPaintLabels(boolean b)  { jSlider.setPaintLabels(b);        }

    public boolean getPaintLabels()        { return jSlider.getPaintLabels();  }

    public void setPaintTicks(boolean b)   { jSlider.setPaintTicks(b);         }

    public boolean getPaintTicks()         { return jSlider.getPaintTicks();   }

    public void setPaintTrack(boolean b)   { jSlider.setPaintTrack(b);         }

    public boolean getPaintTrack()         { return jSlider.getPaintTrack();   }

    public void setSnapToTicks(boolean b)  { jSlider.setSnapToTicks(b);        }

    public boolean getSnapToTicks()        { return jSlider.getSnapToTicks();  }

    public void setInitValue(int val)      { jSlider.setValue(val);
                                             jLabel.setText( (new Double(val)).toString());
                                           }

    public int getInitValue()              { return jSlider.getValue();        }





}
