
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class CollectionPaintView     -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.output.*;
import javax.swing.*;

/**
* Animation view to visualize a vector of <code>Component</code> objects. 
* Paints all <code>Component</code> objects by adding it to the view 
* (is <code>Container</code>).
* <p>
* Use the view by adding it as VariableChangeListener to a <code>VectorVariable</code>
* <p>
* Costumization: 
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set ascending or descending order by using  <code>setAscending</code> or <code>setDescending</code>
* <li> set alignment for arrangement of objects by <code>setAlignment</code> (using FlowLayout constants).
* </ul>
* @see sim.kernel.VectorVariable
*
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class CollectionPaintView extends VariableView {

  /** Default constructor CollectionPaintView */
  public CollectionPaintView() {
    setLayout(new FlowLayout());
  }

  /** Gets the minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
    return new Dimension(70,10);
  }

// ascending/descending ------------------------------------------------------------------------------

  /** variable to store the ascending/descending alignment */
  protected boolean ascending= true;

  /** Sets the ascending/descending alignment of the painted objects */
  public void setAscending(boolean a) {
     ascending= a;
      repaint();
   }

   /** Sets the ascending/descending flag of the painted objects */
  public void setDescending(boolean d) {
      ascending = ! d;
      repaint();
   }

  /** Tests if the alignment is ascending */
  public boolean isAscending() {
     return ascending;
   }

  /** Tests if the alignment is ascending */
  public boolean isDescending() {
     return ! ascending;
   }


// alignment ----------------------------------------------------------------------------------------------

  /** variable to store the alignment */
  int alignment = FlowLayout.LEFT;

  /** Gets the alignment */
  public int getAlignment() {
     return alignment;
   }

  /** Sets the alignment */
  public void setAlignment(int a) {
     alignment = a;
   }

  /**
    * Reacts to a new property value by adding all objects in vector to view.
    * Called by super.propertyChange. 
    * @param newVal The new vector object received from the property change.
    */
    public void setValue (Object newVal) {
        try {
            super.setValue (newVal);
            removeAll();
            Collection collectionValue = (Collection)getValue();
            Iterator i = collectionValue.iterator();
            if (collectionValue != null) {
                while (i.hasNext()) {
                    add((Component) i.next());
                }
            }
            if (!isAscending()) {
                // reverse the order
            }
            repaint();
        } catch (ClassCastException excpt) {
            System.out.println("Not an appropriate value in Variable detected " + getValue()); 
        }
    }

  /** Empty  because painting of added components is done by Container.paint */
  public void paintValue(Graphics g) {
  }

}
