
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ProgressView         --------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.output;

import java.awt.*;
import java.util.*;
import java.beans.*;

/**
* Animation view to visualize a numeric value by a progress bar. 
* Can serve as variable change listeners for numeric variables.
* Paints the numeric value by drawing a bar of appropriate length in the view.  
* <p>
* Use the view by adding it as VariableChangeListener to a numeric variables. 
* <p>
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set the maximum represented value in the bar by using <code>setMaxNr</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class ProgressView extends NumberView {

  /** Default constructor ProgressView */
  public ProgressView() {
  }

// size  -----------------------------------------------------------------------------------------------------

  /** Gets the minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
    return new Dimension(70,20);
  }


// maxNr -----------------------------------------------------------------------------------------------------

  /**
  * Variable to store a maximum number represented in the progress bar.
  * The bar displayed has its maximum height at this value.
  */
  int maxNr= 10;
  /** Sets the maximum number shown by the bar. */
  public void setMaxNr(int n) { maxNr= n; repaint(); }
  /** Gets the maximum number shown by the bar. */
  public int getMaxNr() { return maxNr; }

    
// PaintValue ------------------------------------------------------------------------------------------------

  /**
  * Paints the numeric value by drawing a bar of appropriate length.
  * @param g the graphics context of the view
  */
  public void paintValue(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    int val;
    if (getNumber() == null)
        val = 0;
    else
        val = getNumber().intValue();
    if (maxNr>0) {
      int currLen= width*val/maxNr;
      if (val>0) {
          g.setColor(getForeground());
          g.fill3DRect(3, 3, currLen, height-17, true);
      }
    }
  }
}
