
/**
 * VariableChangeListenerAlarm.java
 *
 *
 * Created: Wed Nov 17 23:27:57 1999
 *
 * @author Reitinger Bernhard
 * @version 1.0
 */
package mosaic.sim.kernel.output;

import java.util.ArrayList;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;

abstract public class VariableChangeListenerAlarm extends AbstractVariableChangeListener
         implements AlarmEventComponent, VariableChangeListener {
    private ArrayList listenerList;
    private boolean status = false;
    private String message;
    

    public VariableChangeListenerAlarm() {
    }

    /**
       * Get the value of message.
       * @return Value of message.
       */
    public String getMessage() {return message;}
    
    /**
       * Set the value of message.
       * @param v  Value to assign to message.
       */
    public void setMessage(String  v) {this.message = v;}
    
    /**
       * Get the value of status.
       * @return Value of status.
       */
    public boolean getStatus() {return status;}
    
    /**
       * Set the value of status.
       * @param v  Value to assign to status.
       */
    public void setStatus(boolean  v) {this.status = v;}
   

    // Interface VariableChangeListener ------------------------------------------------

    /** Reacts to a variable changed event. */
    public void variableChange(VariableChangeEvent e) {
	if (!status) {
	    if (testCondition(e)) {
 		fireAlarmSet();
	    }
	} else {
	    if (!testCondition(e)) {
		fireAlarmReset();
	    }
	}
    }

    /** tests in the subclass, if alarm should be set */
    abstract public boolean testCondition(VariableChangeEvent e);

    public void fireAlarmSet() {
	if (listenerList != null) {
	    for (int i=0; i<listenerList.size(); i++) {
		((AlarmEventListener)listenerList.get(i)).alarmSet(new AlarmEvent(this, message, status));
	    }
	}
	status = true;
    }
    
    public void fireAlarmReset() {
	if (listenerList != null) {
	    for (int i=0; i<listenerList.size(); i++) {
		((AlarmEventListener)listenerList.get(i)).alarmReset(new AlarmEvent(this, message, status));
	    }	
	}
	status = false;
    }
    
  /** Reacts to a variable changed event. */
    public void propertyChange (java.beans.PropertyChangeEvent e) {}

  /** Adds a listener variable. */
    public void addListenerVariable (Variable v) {}

  /** Removes a listener variable. */
    public void removeListenerVariable (Variable v) {}


    // adding listeners -----------------------------------------------------

    public void addAlarmEventListener(AlarmEventListener l) {
	if (listenerList == null) listenerList = new ArrayList();
	if (!listenerList.contains(l)) listenerList.add(l);
    }

    public void removeAlarmEventListener(AlarmEventListener l) {
	if (listenerList != null && listenerList.contains(l)) {
	    listenerList.remove(l);
	    listenerList.trimToSize();	
	}
    }
    
} // VariableChangeListenerAlarm
