
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class NormalDistribution         --------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 29.12.98
// documented by: hp, 29.12.98

package mosaic.sim.kernel.random;

import java.lang.Math;

/**
* Class realizing a normal distribtion.
* Has as parameters the mean and the standard deviation.
* <p> 
* <b>Customization </b>
* <ul>
* <li>Use setMean to set the mean of the normal distribution 
* <li>Use setDeviation to set the standard deviation of the normal distribution 
* </ul>
*/
public class NormalDistribution extends RandomDistribution 
{
  /** Constructor intializing mean by 1 and standard deviation by 0. */
  public NormalDistribution () {
        setMean (1.0);
        setDeviation (0.0);
        setLabel("nD");
  }

  /** Constructor  with initialization paramters for mean and standard deviation.  */
  public NormalDistribution (double iDeviation, double iMean) {
        setMean (iMean);
        setDeviation (iDeviation);
        setLabel("nD");
  }

  /** Sets the mean of the distribution */
  public void setMean (double aMean) {
        mean = aMean;
  }

  /** Gets the mean of the distribution */
  public double getMean () {
        return mean;
  }

  /** Sets the standard deviation of the distribution */
  public void setDeviation (double aDeviation) {
        deviation = aDeviation;
  }

  /** Gets the standard deviation of the distribution */
  public double getDeviation () {
        return deviation;
  }

  /** Generates a random integer from the normal distribution  */
  public int getRandInt()  {
        double sum = 0;
        for (int i = 1; i <= 12; i++) {
            sum = sum + randomSeed.nextDouble();
        }
        return ((int) Math.round(deviation * (sum - 6) + mean));
  }

  /** Generates a random double from the normal distribution  */
  public double getRandDouble()  {
        double sum = 0;
        for (int i = 1; i <= 12; i++) {
            sum = sum + randomSeed.nextDouble();
        }
        return (deviation * (sum - 6) + mean);
  }

  /** variables to hold mean  */
  protected double mean;

  /** variables to hold  standard deviation */
  protected double deviation;


}