package mosaic.sim.kernel.random;
import java.beans.*;
import java.awt.*;
import mosaic.util.custom.*;


public class TimeSeedEditor extends java.beans.PropertyEditorSupport
  implements CustomTagsPropertyEditor
{
    /** array with switches to control runtime debug output*/
    private boolean[] DEBUG = new boolean[] {false};

    /** non - argument constructor */
    public TimeSeedEditor() {
      if (DEBUG[0]) OutFrame.out("newInstance created - " + this);
      listeners = new PropertyChangeSupport(this);
    }

    /** returns only the ClassName*/
    public String getAsText ()
    {
        Object val = getValue();
        // .getName() returns "sim.kernel.util.*", so .equals("*") failed.
        //OutFrame.out ("getAsText " + this);
          if (val.getClass().getName().endsWith ("ConstantTimeSeed"))
              return "ConstantTimeSeed";
          else if (val.getClass().getName().endsWith ("TimeSequence"))
              return "TimeSequence";
          else if (val.getClass().getName().endsWith ("UniformDistribution"))
              return "UniformDistribution";
          else if (val.getClass().getName().endsWith ("NormalDistribution"))
                return "NormalDistribution";
          else if (val.getClass().getName().endsWith ("ExponentialDistribution"))
              return "ExponentialDistribution";
          else return ("unknown !"); //warning !
     }

    /** never returns null */
    public Object getValue() {
      if (DEBUG[0]) OutFrame.out(this.toString() + " GETVALUE " + super.getValue());
        Object val;
        if (super.getValue() == null) {
            OutFrame.out("new Expodist - " + this);
            val = new ExponentialDistribution();
            setValue(val);
        }
        return super.getValue();
    }

    /** if the parameter is null, a new ExponentialDIstribution is created [PENDING]*/
    public void setValue(Object value) {
      if (DEBUG[0]) OutFrame.out(this.toString() + " SETVALUE " + value);
      if (value != null) super.setValue(value);
      else super.setValue(new ExponentialDistribution());
      //OutFrame.out("SetValue"+ value + this);
      fire();
    }

    /** creates new Objects using no argument contructors */
    public void setAsText(String text)
            throws java.lang.IllegalArgumentException
    {
        if (text.equals ("ConstantTimeSeed")) setValue (new ConstantTimeSeed());
        else if (text.equals ("UniformDistribution")) setValue (new UniformDistribution());
        else if (text.equals ("TimeSequence")) setValue (new TimeSequence());
        else if (text.equals ("NormalDistribution")) setValue (new NormalDistribution());
        else if (text.equals ("ExponentialDistribution")) setValue (new ExponentialDistribution());
        else throw (new IllegalArgumentException());
        //OutFrame.out("SetAsTExt "+ this);
    }

    /**implements interface CustomPropertyEditor*/
    public boolean supportsCustomTags() {
      return true;
    }

    /**implements interface CustomPropertyEditor*/
    public String[] getCustomTags()
    {
        return new String[] {"ConstantTimeSeed",
                             "TimeSequence",
                             "UniformDistribution",
                             "NormalDistribution",
                             "ExponentialDistribution"};
    }
    /** uses an Initstringcreator which uses the only the value - object */
    public String getJavaInitializationString()
    {
        OutFrame.out("Java init called "+this);
        InitStringCreator aISC = new InitStringCreator();
           return aISC.getInitStringForBean(getValue());
    }

    /*public void updateJavaInitString() {
        /** This returns: "new TYPENAME ( <parmString from CPEV2> );"
         *  CPEV2 does not know anything about the TYPENAMEs used here
         *  it only uses propertydescriptor - information
         *  from the BeanInfo Object

        Object val = getValue();
        String parmString;
        if (customEditor != null) {
            parmString = customEditor.getJavaActualParameterString();
            if (val.getClass().getName().endsWith ("UniformDistribution"))
                javaInitString = ("new UniformDistribution(" + parmString + ")");
            else if (val.getClass().getName().endsWith ("NormalDistribution"))
                javaInitString = "new NormalDistribution( " + parmString +  " )";
            else if (val.getClass().getName().endsWith ("ExponentialDistribution"))
                  javaInitString = "new ExponentialDistribution( " + parmString +" )";
              else if (val.getClass().getName().endsWith ("RandomDistribution"))
                javaInitString = "new RandomDistribution( "+ parmString + " )";
            else javaInitString = "null"; //warning !
            OutFrame.out("HaveEditor");
        }
        OutFrame.out("update " + javaInitString + " - " + this);
    }
   */

   /** declares that there is a custom editor available */
   public boolean supportsCustomEditor()
    {
        return true;
    }

    /** */
    public java.awt.Component getCustomEditor()
    {
        customEditor = new PropertyCustomEditor(this);
        if (DEBUG[0]) OutFrame.out("GotCustomEditor - " + this);
        return (customEditor);
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
      if (DEBUG[0]) OutFrame.out("Add PropertyChangeListener");
      listeners.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
      listeners.removePropertyChangeListener(listener);
    }
    public void fire() {
      listeners.firePropertyChange("Value", null, getValue());
      if (DEBUG[0]) OutFrame.out("fire ! " + this);
    }

    private PropertyCustomEditor customEditor;
    private PropertyChangeSupport listeners;

}
