
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class UniformDistribution        --------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 29.12.98
// documented by: hp, 29.12.98

package mosaic.sim.kernel.random;

import java.lang.Math;

/**
* Class realizing a uniform distribution.
* Has as parameters the low and high values.
* <p> 
* <b>Customization </b>
* <ul>
* <li>Use setLow to set the lower value of the uniform distribution 
* <li>Use setHigh to set the higher value of the uniform distribution 
* </ul>
*/
public class UniformDistribution extends RandomDistribution 
{

  /** Constructor intializing low by 0.0 and high by 1.0. */
  public UniformDistribution () {
        setLow (0.0);
        setHigh (1.0);
        setLabel("uD");
  }

  /** Constructor intializing with initialization paramters for low and high.  */
  public UniformDistribution (double iHigh, double iLow) {
        setLow (iLow);
        setHigh (iHigh);
        setLabel("uD");
  }

  /** Sets the low value of the distribution */
  public void setLow (double aLow) {
        low = aLow;
  }

  /** Gets the low value of the distribution */
  public double getLow () {
        return low;
  }

  /** Sets the high value of the distribution */
  public void setHigh (double aHigh) {
        high = aHigh;
  }

  /** Gets the high value of the distribution */
  public double getHigh () {
       return high;
  }

  /** Generates a random integer from the uniform distribution  */
  public int getRandInt()  {
        return ((int)Math.round(randomSeed.nextDouble() * (high - low) + low));
  }

  /** Generates a random double from the uniform distribution  */
  public double getRandDouble()  {
        return (randomSeed.nextDouble() * (high - low) + low);
  }

  protected double high;protected double low;

}