// Generated by Together  
// Abstract base class provides basic implementation for InputEventComponents 

package mosaic.sim.kernel.simevent;

import mosaic.sim.kernel.control.*;
import java.util.Vector;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.*;
import mosaic.sim.kernel.AbstractModelComponent;
import mosaic.sim.kernel.input.*; 

public class BasicInputEventComponent extends AbstractModelComponent 
	implements UserEventListener, InputEventComponent {

    /** contains all InputEventListeners */
   protected Vector listeners = new Vector();

   /** contains object holding userInput */
   protected Object userInput;

   public BasicInputEventComponent()
   {
      this ("BasicInputEventComponent");
   }

   /** BasicInputEventComponent constructor with label*/
   public BasicInputEventComponent(String iLabel) {
     super(iLabel);
     setForeground(Color.red);
     setFont(new Font("SansSerif", Font.PLAIN, 10));
    }


// tL ----------------------------------------------------------------------------------------------

   /**
   * Gets the time of last event
   */
   public double getTL()
      { return 0.0; }

   /**
   * Sets the time of last event
   */
   public void setTL(double newTime)
      { ; }

// -----------------------------------------------------------------------------------------------------------
// interface Comparable --------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

  /** compares this timer with another timer c based on tn and selectOrder */
  public boolean before(Comparable c) {
    if (c instanceof SimEventComponent) {
      SimEventComponent s = (SimEventComponent) c;
      return getSelectOrder()>s.getSelectOrder();
    }
    return false; //use exception!!
  }

// select order  ---------------------------------------------------------------------------------------------

  /** variable to hold the selectOrder for the timer which is used as a priority
  * for tie breaking between several timer with equal next event time.
  */
  protected int selectOrder = 0;

  /** Gets the select order of the timer */
  public int getSelectOrder() {
      return selectOrder;
  }

  /** Sets the select order of the timer */
  public void setSelectOrder(int newOrder) {
    int oldSelectOrder = selectOrder;
    selectOrder= newOrder;
  }


// event -------------------------------------------------------------------------------------------

   /**
   * Trigger an event
   */
     public void triggerEvent() {
      InputEvent inputEvent = new InputEvent(this);
      Vector listenersCopy = (Vector)(listeners.clone());
      for(int loop=0;loop<listenersCopy.size();loop++)
         ((SimEventListener)listenersCopy.get(loop)).processSimEvent(inputEvent);
	    }

// access to input object --------------------------------------------------------------------------

  public Object getInput() {
    return userInput;
  }


// add and remove listeners ------------------------------------------------------------------------

   /**
   * Add SimEventListener
   */
   public void addSimEventListener(SimEventListener simEventListener) {
      if (!listeners.contains(simEventListener))
         listeners.add(simEventListener);
      }

	 /**
   * Remove SimEventListener
   */
   public void removeSimEventListener(SimEventListener simEventListener) {
      if (listeners.contains(simEventListener))
         listeners.remove(simEventListener);
      }

   /**
   * Add InputEventListener
   */
   public void addInputEventListener(InputEventListener inputEventListener) {
     if (!listeners.contains(inputEventListener))
         listeners.add(inputEventListener);
   }

	 /**
   * Remove InputEventListener
   */
   public void removeInputEventListener(InputEventListener inputEventListener) {
      if (listeners.contains(inputEventListener))
         listeners.remove(inputEventListener);
      }

// paint -----------------------------------------------------------------------------------------------------

  /* Paints the BasicInputEventComponent */
  public void paintModelComponent(Graphics g) {
      int h = getSize().height;
      int w = getSize().width;
      g.setColor(getForeground());
      g.drawRect(1, h/3, w-2, h/3);
      g.drawPolygon( new int[]{w/4, w/2, 3*w/4}, new int[]{0, h/3, 0}, 3);
      g.drawPolygon( new int[]{w/4, w/2, 3*w/4}, new int[]{2*h/3, h, 2*h/3}, 3);
      g.setFont(getFont());
      FontMetrics fm = g.getFontMetrics();
      g.setColor(getForeground());
      String lbl = getLabel();
      if (lbl == null) {
          lbl = "V";
      }
      g.drawString(lbl,2,(h+fm.getMaxDescent())/2);
  }

//size: ------------------------------------------------------------------------------------------------------

  /** Gets the minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
    return new Dimension(5,5);
  }

  /** Gets the preferred size for the LayoutManager */
  public Dimension getPreferredSize() {
    return new Dimension(16,12);
  }


//debug: -----------------------------------------------------------------------------------------------------

  /** Generates a info string for the timer */
  public String getInfo() {
    String text = getLabel() + ":" + getDebugText();
    return text;
  }

  /** Generates a debug string for the timer */
  protected String getDebugText() {
    return "inputEvent";
  }

// -----------------------------------------------------------------------------------------------------------
// interface UserEventListener -------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

  /** Reacts to UserEvents from userEventComponents, get Object from userEventComponent to
 *       forward it at next triggerEvent.
  */
  public void processSimEvent (SimEvent evt) {
      getSimApplication().getSimulator().addInputEvent(this);
      if (evt instanceof UserEvent)
        userInput = ((UserEventComponent)(evt.getSource())).getUserInput();
  }

  /** variable to hold the SimEventComponents */
  Vector simEventComponents = new Vector ();

  /** Adds a SimEventComponent. Is called from addSimEventListener of SimEventComponent. */
  public void addSimEventComponent (SimEventComponent c){
    if (! simEventComponents.contains(c))
      simEventComponents.add(c);
  }

  /** Removes a SimEventComponent. Is called from removeSimEventListener of SimEventComponent.  */
  public void removeSimEventComponent (SimEventComponent c){
    if (! simEventComponents.contains(c))
      simEventComponents.add(c);
  }


   /** Adds a UserEventComponent. Is called from addUserEventListener of UserEventComponent. */
   public void addUserEventComponent (SimEventComponent c) {
      addSimEventComponent(c);
   }

   /** Removes a UserEventComponent. Is called from removeUserEventListener of UserEventComponent.  */
   public void removeUserEventComponent (SimEventComponent c){
      removeSimEventComponent(c);
   }    

}
                                                          







