package mosaic.sim.kernel.statistics;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.control.*;

/**
* Class for computing observational (not time weighted) statistics over time durations.
* Extends ObservationalStatistics.
* Measures time durations between method calls of begin() and end()
* and computes the statistics of the duration between.
* Used in sim.itemproc.statistics.ItemEventStatistics.
* Computes max, min, mean and std devition of the values received per observation.
* Has the capability to open a report window when clicked on the bean and give out the current statistics.
* <p>
*
* @author Herbert Praehofer
* @version %I% %G%
*/
public class DurationStatistic extends ObservationalStatistic
{

  /** Default constructor StatisticsBean */
  public DurationStatistic() {
  }

// begin and end of duration interval ------------------------------------------------------------------------

  /** Marks the beginning of a duration. */
  public void begin () {
      setBeginTime (getTime());
  }

  /** Marks the end of a duration. */
  public void end () {
      setDuration (getBeginTime() - getTime());
      newObservation(new Double (getDuration()));
  }

// begin time ------------------------------------------------------------------------------------------------

  protected double beginTime;

  /** Gets the begin time */
  public double getBeginTime () {
      return beginTime;
  }

  /** Sets the begin time */
  protected void setBeginTime (double bt) {
      beginTime = bt;
  }



// duration variable -----------------------------------------------------------------------------------------

  protected double duration;

  /** Gets the duration time */
  public double getDuration() {
      return duration;
  }

  /** Sets the duration time interval */
  protected void setDuration (double d) {
      duration = d;
  }


// reset -----------------------------------------------------------------------------------------------------

  /** Clears the statistics */
  public void clearStatistics() {
      super.clearStatistics();
      setDuration(0);
  }


}

