/*
 *  CharVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;


/**
* CharVariable is a Variable for values of type char.
* Use this bean class to realize simulation model variables of type char.
* Inherits the features of Variable, in particular: to signal variableChangeEvents.
* Has a char value and <code>getValue</code> and <code>setValue</code> to read and write it. 
*
* @see sim.kernel.Variable
*/
public class CharVariable extends mosaic.sim.kernel.variable.AbstractVariable {

// Constructors ------------------------------------------------------------------------------------


  /** Default constructor for BooleanVariable*/
   public CharVariable() {
        this("char", VARIABLE, "", "", "", 'a');
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public CharVariable (char value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public CharVariable (String name, char value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public CharVariable (String name, int type, char value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public CharVariable(String name, int type,
                    String unit, String quantity, String displayUnit, char value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// char value      ---------------------------------------------------------------------------------

  /** field to hold the value of the variable */
  protected char value = 'a';

  /** Gets the char value of the variable */
  public final char getValue () {
      return ((CharVariable)definedBy).value;
  }

  /** Sets the char value of the variable */
  public final void setValue (char newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the value of the variable as type Object */
  public final Object getValueObject () {
      return new Character(getValue());
  }

// overwrite method processSimEvent from interface InputEventListener  -----------------------------

  /** Reacts to SimEvents from SimEventComponents.
   *  The events should be triggered by a inputEventSource and
   *  return Character-Objects.
   */
  public void processSimEvent (SimEvent evt){
    Object obj;
    if ((evt.getSource()) instanceof InputEventComponent){
      obj = ((InputEventComponent)(evt.getSource())).getInput();
      if (obj instanceof Character)
        this.setValue( ((Character)(obj)).charValue() );
    }
  }

}

