/*
 *  CollectionVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;
import java.util.*;

/**
 * Abstract class for all Variables which function like a collection.
 */
abstract public class CollectionVariable
   extends mosaic.sim.kernel.variable.AbstractVariable //++ not compatible with Component: implements Collection
{

  /** Constructor for CollectionVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type. */
  public CollectionVariable(String name, int type,
                    String unit, String quantity, String displayUnit) {
      super(name, type, unit, quantity, displayUnit);
   }


// getValueAsCollection

    /** Gets the value as Collection object */
    abstract public Collection getValueAsCollection();

//
// nrElements IntVariable     ----------------------------------------------------------------------
//
//
// maxNr  ------------------------------------------------------------------------------------------
//

  /** int variable maxNr determining the maximum elements in the Vector */
  protected int maxNr = Integer.MAX_VALUE;

  /** Gets the maximal number of elements in this Vector variable */
  public int getMaxNr () {
      return maxNr;
  }

  /** Sets the maximal number of elements in this Vector variable */
  public void setMaxNr (int newMaxNr) {
      maxNr = newMaxNr;
  }


// Collection interface implementation -------------------------------------------------------------
// All requests are forwarded to the value value.
// In all methods which will change the value, a variableChange is signaled.

    public int _size() {
        return getValueAsCollection().size();
    }

    public boolean _isEmpty() {
        return getValueAsCollection().isEmpty();
    }

    public boolean _contains(Object o){
        return getValueAsCollection().contains(o);
    }

    public Iterator _iterator(){
        return getValueAsCollection().iterator();
    }

    public Object[] _toArray() {
        return getValueAsCollection().toArray();
    }

    public Object[] _toArray(Object[] a){
        return getValueAsCollection().toArray(a);
    }


    public boolean _add(Object o) {
        Collection coll = getValueAsCollection();
        if (coll.add(o)) {
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public boolean _remove(Object o) {
        Collection coll = getValueAsCollection();
        if (coll.remove(o)) {
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }


    public boolean _containsAll(Collection c) {
        return getValueAsCollection().containsAll(c);
    }

    public boolean _addAll(Collection c) {
        Collection coll = getValueAsCollection();
        if (coll.addAll(c)) {
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public boolean _removeAll(Collection c) {
        Collection coll = getValueAsCollection();
        if (getValueAsCollection().removeAll(c)) {
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public boolean _retainAll(Collection c) {
        Collection coll = getValueAsCollection();
        if (coll.retainAll(c)) {
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public void _clear(){
        Collection coll = getValueAsCollection();
        coll.clear();
        signalVariableChange();
    }

    public boolean _equals(Object o) {
        return getValueAsCollection().equals(o);
    }

}

