/*
 *  IntVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;

import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;

/**
* IntVariable is a Variable for values of type int.
* Use this bean class to realize simulation model variables of type int.
* Has a int value and <code>getValue</code> and <code>setValue</code> to read and write it.
* Inherits the features of Variable, in particular: to signal variableChangeEvents.
*
* @see sim.kernel.Variable
*/
public class IntVariable extends mosaic.sim.kernel.variable.AbstractVariable {

// Constructors ------------------------------------------------------------------------------------


  /** Default constructor for BooleanVariable*/
   public IntVariable() {
        this("", VARIABLE, "", "", "", 0);
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public IntVariable (int value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public IntVariable (String name, int value) {
        this( name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public IntVariable (String name, int type, int value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public IntVariable(String name, int type, String unit, String quantity, String displayUnit, int value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// int value      ----------------------------------------------------------------------------------

  /** field to hold the value of the variable */
  protected int value = 0;

  /** Gets the int value of the variable */
  public final int getValue () {
    return ((IntVariable)definedBy).value;
  }

  /** Sets the int value of the variable */
  public final void setValue (int newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// getValueObject      -----------------------------------------------------------------------------

  /** Gets the value of the variable as type Object */
  public final Object getValueObject () {
      return new Integer(getValue());
  }

// overwrite method processSimEvent from interface InputEventListener  -----------------------------

  /** Reacts to SimEvents from SimEventComponents.
   *  The events should be triggered by a inputEventComponent and
   *  return Number-Objects.
   */
  public void processSimEvent (SimEvent evt){
  Object obj;
    if ((evt.getSource()) instanceof InputEventComponent){
      obj = ((InputEventComponent)(evt.getSource())).getInput();
      if (obj instanceof Integer)
        this.setValue( ((Integer)(obj)).intValue() );
    }
  }


}

