/*
 *  ListVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;
import java.util.*;

/**
 * Abstract class for all Variables which function like a List.
 */
abstract public class ListVariable
   extends mosaic.sim.kernel.variable.CollectionVariable //++ not compatible with Component: implements List
{

// Constructors ------------------------------------------------------------------------------------

  /** Constructor for ListVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type. */
  public ListVariable(String name, int type,
                    String unit, String quantity, String displayUnit) {
      super(name, type, unit, quantity, displayUnit);
   }

// getValueAsList
    /** Gets the value as List object */
    abstract public List getValueAsList ();


// List interface implementation -----------------------------------------------------------------
// All requests are forwarded to the value value.
// In all methods which will change the value, a variableChange is signaled.


    public Object _get(int index) {
        return getValueAsList().get(index);
    }

    public Object _set(int index, Object element) {
        Object o = getValueAsList().set(index, element);
        signalVariableChange();
        return o;
    }

    public void _add(int index, Object element) {
        getValueAsList().add(index, element);
        signalVariableChange();
    }

    public Object _remove(int index) {
        Object obj = getValueAsList().remove(index);
        signalVariableChange();
        return obj; 
    }

    public int _indexOf(Object o) {
        return getValueAsList().indexOf(o);
    }

    public int _lastIndexOf(Object o) {
        return getValueAsList().lastIndexOf(o);
    }

    public ListIterator _listIterator() {
        return getValueAsList().listIterator();
    }

    public ListIterator _listIterator(int index) {
        return getValueAsList().listIterator(index);
    }

    public List _subList(int fromIndex, int toIndex) {
        return getValueAsList().subList(fromIndex, toIndex);
    }

}

