/**
 * Project mosaic
 *
 * PropertyChangeRegistry.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel.variable;

import java.lang.*;
import java.lang.reflect.*;
import java.beans.*;
import mosaic.sim.kernel.ModelComponent;

/**
* Class for registration of property change events.
* <p>
* For each property change event such an object is created and queue gobally in the simulator. 
* The registries are handled in fifo manner by the simulator by accessing the property value through 
* the get function of the property (propGetFnName) and sending an PropertyChangeEvent to the listener l. 
*/

public class PropertyChangeRegistry extends mosaic.sim.kernel.variable.ChangeRegistry {

// fields ------------------------------------------------------------------------------------------

    /** variable to hold the listener for the property change event */
    PropertyChangeListener listener;
    /** variable to hold the name of the property change event */
    String propertyName;
    /** variable to hold the access method for the property */
    Method method;

// Constructors ------------------------------------------------------------------------------------

    /** 
    * Constructor for the PropertyChangeRegistry initializing source, property name, the name of the 
    * get function of the property, and the listener. 
    * Gets the get method from the name through the reflection package. 
    */
    public PropertyChangeRegistry (ModelComponent source, String property,
                                   String propGetFnName, PropertyChangeListener l) {
        super(source);
        propertyName = property;
        listener = l;
        try {
            method = source.getClass().getMethod(propGetFnName, null);
        }
        catch (NoSuchMethodException e) {}
    }

//Overloaded methods to make an object of all data types -------------------------------------------

    /** Returns the object val  unchanged. */
    static protected Object objectify (Object val) {
        return val;
    }
    /** Returns the Boolean object for boolean value. */
    static protected Object objectify (boolean val) {
        return (new Boolean (val));
    }
    /** Returns the Character object for char value. */
    static protected Object objectify (char val) {
        return (new Character  (val));
    }
    /** Returns the Byte object for byte value. */
    static protected Object objectify (byte val) {
        return (new Byte (val));
    }
    /** Returns the Double object for double value. */
    static protected Object objectify (double val) {
        return (new Double (val));
    }
    /** Returns the Float object for float value. */
    static protected Object objectify (float val) {
        return (new Float (val));
    }
    /** Returns the Integer object for int value. */
    static protected Object objectify (int val) {
        return (new Integer (val));
    }
    /** Returns the Long object for long value. */
    static protected Object objectify (long val) {
        return (new Long (val));
    }
    /** Returns the Short object for short value. */
    static protected Object objectify (short val) {
        return (new Short  (val));
    }

// fireChangeEvent ---------------------------------------------------------------------------------

    /** 
    * Executes a property change event.  
    * Accesses the property value by the get method, objectifies the value,
    * and sends a property change event to the listener. 
    */
    public   void fireChangeEvent () {
        try {
            Object propValue = objectify (method.invoke (source, null));
            listener.propertyChange (new PropertyChangeEvent (source, propertyName, null, propValue));
        }
        catch (InvocationTargetException e) {}
        catch (IllegalAccessException e) {}
    }

} // end PropertyChangeRegistry ====================================================================

