/*
 *  TreeSetVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.util.*;
import mosaic.sim.kernel.ModelComponent;

/**
* Class TreeSetVariable is a Variable for values of type TreeSet.
* Use this bean class to realize simulation model variables of type TreeSet.
* Has a TreeSet value and <code>getValue</code> and <code>setValue</code> to read and write the vector.
* Additionally provides all common List methods to access the members of the TreeSet, e.g. add,
* remove, etc. and signals a change whenever the vector changes.
* <p>
* Additionally provides the number of elements in the TreeSet (nrElements) and allows to
* add and remove VariableChangeListeners to nrElements (addNrElementsListener, removeNrElementsListener).
*
* @see sim.kernel.Variable
*/
public class TreeSetVariable
   extends mosaic.sim.kernel.variable.SortedSetVariable {

// Constructors ----------------------------------------------------------------------------------------------

  /** Default constructor for BooleanVariable*/
   public TreeSetVariable() {
        this("", VARIABLE, "", "", "");
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public TreeSetVariable (ModelComponent parent, String name) {
        this(name, VARIABLE, "", "", "");
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public TreeSetVariable (String name, int type) {
        this(name, type, "", "", "");
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public TreeSetVariable(String name, int type,
                    String unit, String quantity, String displayUnit) {
      super(name, type, unit, quantity, displayUnit);
      this.value = new TreeSet(FIFO);
   }

// TreeSet value ---------------------------------------------------------------------------------

  /** field to hold the vector of the variable */
  protected TreeSet value;

  /** Gets the Object vector of the variable */
  public final TreeSet getValue () {
    return ((TreeSetVariable)definedBy).value;
  }

  /** Sets the Object vector of the variable */
  public final void setValue (TreeSet newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// sorting criteria --------------------------------------------------------------------------------

    /** Sets a new sorting criteria for the sorted set*/
    public void setSorting(Comparator comparator) {
        if (! sortingCriteria.equals(comparator)) {
            sortingCriteria = comparator;
            TreeSet newSet;
            if (sortingCriteria == null) {
                newSet = new TreeSet();
            } else {
                newSet = new TreeSet(sortingCriteria);
            }
            newSet.addAll(value);
            setValue (newSet);
        }
    }

    /** Gets the sorting criteria for the sorted set*/
    public Comparator getSorting() {
        return sortingCriteria;
    }

// getValueObject      -----------------------------------------------------------------------------

  /** gets the vector of the variable as type Object */
  public final Object getValueObject () {
      return ((TreeSetVariable)definedBy).value;
  }

// getValueAsSet
    /** Gets the value as List object */
    public final Set getValueAsSet () {
        return ((Set) value);
    }

// getValueAsSortedSet
    /** Gets the value as List object */
    public final SortedSet getValueAsSortedSet () {
        return ((SortedSet) value);
    }

// getValueAsCollection
    /** Gets the value as List object */
    public final Collection getValueAsCollection () {
        return ((Collection) value);
    }

}
