/**
 * Project mosaic
 *
 * VariableChangeEvent.java
 *
 * copyright 2000, JKU Linz, Austria
 *
 */

package mosaic.sim.kernel.variable;

import java.beans.PropertyChangeEvent;

/**
* Event object for variable change events.
* Extends PropertyChangeEvent.
* Carries the variable, model, time, and the value of the variable change.
* Used in similar way as PropertyChangeEvent of java.beans.
*/
public class VariableChangeEvent extends PropertyChangeEvent {


// Constructors ------------------------------------------------------------------------------------

    /** Constructors for VariableChangeEvent initializing event source */
    public VariableChangeEvent (mosaic.sim.kernel.variable.Variable iVariable) {
    	this (iVariable, null, 0.0);
    }

    /** Constructors for VariableChangeEvent initializing event source and model */
    public VariableChangeEvent (mosaic.sim.kernel.variable.Variable iVariable, Object newValue) {
     	this (iVariable, newValue, 0.0);
   }

    /** Constructors for VariableChangeEvent initializing event source, model, variable and value */
    public VariableChangeEvent (mosaic.sim.kernel.variable.Variable iVariable, Object newValue, double changeTime) {
        super(iVariable, iVariable.getLabel(), null, newValue);
        setVariable (iVariable);
        setValue (newValue);
        setChangeTime(changeTime);
    }


// value  ------------------------------------------------------------------------------------------

    /** variable to hold the value of the variable change event.  */
    private Object value;

    /** Gets the value of the variable change event. */
    public Object getValue () {
        return value;
    }

    /** Sets the value of the variable change event. */
    public void setValue (Object newVal) {
        value = newVal;
    }


// variable ----------------------------------------------------------------------------------------

    /** variable to hold the variable of the variable change event.  */
    private mosaic.sim.kernel.variable.Variable variable;

    /** Gets the variable of the variable change event. */
    public mosaic.sim.kernel.variable.Variable getVariable () {
        return variable;
    }

    /** Sets the variable of the variable change event. */
    public void setVariable (mosaic.sim.kernel.variable.Variable newVar) {
        variable = newVar;
    }

    
// change time    ----------------------------------------------------------------------------------

    /** variable to hold the time of the variable change event.  */
    private double changeTime;

    /** Gets the time of the variable change event. */
    public double getChangeTime () {
        return changeTime;
    }

    /** Sets the time of the variable change event. */
    public void setChangeTime (double newTime) {
        changeTime = newTime;
    }

} // end VariableChangeEvent =======================================================================
