/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: Debug.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.10 $
 *  Last changed at: $Date: 2002/05/16 20:43:26 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

/** This class is used for debugging purposes, mainly for printing debug traces dependent on a trace level
 *  and for assertions (which will be supported natively in JDK 1.4 as soon as
 *  it becomes stable).
 */
public class Debug {
	/** Log nothing. */
	public static final int LEVEL_NONE = 0;
	/** Only log critical debug trace statements, this is minimal debugging. */
	public static final int LEVEL_CRITICAL = 1;
	/** Log everything from LEVEL_CRITICAL and error messages. */
	public static final int LEVEL_ERROR = 2;
	/** Log everything from LEVEL_ERROR and warning messages. */
	public static final int LEVEL_WARNING = 3;
	/** Log everything from LEVEL_WARNING and info messages. */
	public static final int LEVEL_INFO = 4;
	/** Log everything from LEVEL_INFO and print various traces for examining
	 *  rhe program paths.
	 */
	public static final int LEVEL_TRACE = 5;
	/** Log everything that can be logged. */
	public static final int LEVEL_ALL = 6;

	/** Debugging methods are enabled in the project only if this is set to
	 *  true.
	 */
	public static final boolean debug = true;
	/** Determines the debugging level, can be one of LEVEL_*. */
	public static final int level = LEVEL_NONE;

	/** Assert that a condition is true.
	 *  @param condition The assertion aborts program execution if condition is
	 *                   false.
	 *  @param location This string should specify the exact program location
	 *                  from where the assertion is called.
	 *  @param message This string is printed out in the debugging trace if the
	 *                 assertion fails.
	 */
	public static void assertCond(boolean condition, String location, String message) {
		if(debug && !condition) {
			System.err.println("Fatal Error: Assertion failed at " + location +
			": '" + message + "'");
		new Exception().printStackTrace();
		System.exit(1);
		}
	}

	/** Output some message if the currently set debugging level is equal to or
	 *  higher than the given minimum debugging level.
	 *  @param minlevel The current debugging level must be equal to or higher
	 *                  than this value for the message to be printed.
	 *  @param message The string that should be printed.
	 */
	public static void output(int minlevel, String message) {
		if (level >= minlevel)
			System.out.print(message);
	}
}
