/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: Potential.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.6 $
 *  Last changed at: $Date: 2001/10/01 18:50:21 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron;

import mosaic.sim.kernel.*;

/** This class represent the potential of a neuron. It is abstract and does not
 *  implement any behaviour, but only defines the interface that the Neuron
 *  class uses to interact with its associated potential.
 */
public abstract class Potential extends ActiveVariable {
    /** Creates a potential object. */
	public Potential () {
		super();
	}

	/** Creates a potential object with a name (mainly useful during
	 *  debugging).
	 */
	public Potential(String name) {
		super(name);
	}

	/** Compute the current Neuron potential from the gradient changes.
	 */
	public abstract double getCurrentPotential();

	/** Merge a new synaptic response from one of the Neuron input synapses
	 *  with the current potential function, resulting in a new potential
	 *  function.
	 */
	public void mergeSynapseResponse(Object change) {
		signalVariableChange();
	}

	/** This implementation of AbstractVariable.getValueObject just returns the
	 *  current object.
	 */
	public Object getValueObject() {
		return this;
	}
}
