/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: MatrixInputCoder.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.1 $
 *  Last changed at: $Date: 2002/04/14 14:42:06 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.netinputs;

import mosaic.sim.neuron.*;

/** This is an interface for giving a two-dimensional matrix of real numbered
 *  values to the network. It must be implemented by a class with a defined
 *  Spike coding scheme (the so-called coder).
 *
 *  @see VectorInputCoder
 */
public interface MatrixInputCoder {
	/** Enters a new real numbered input matrix into the simulation.
	 *  @param vector The input vector to be used.
	 */
	public void enterMatrix(double[][] matrix) throws CodingException;

	/** Returns the output with the given indices.
	 *
	 *  @param dim1 The index of the output in dimension 1.
	 *  @param dim2 The index of the output in dimension 2.
	 */
	public NeuronOutput getOutput(int dim1, int dim2);
}
