/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: RateCodedInputVector.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.2 $
 *  Last changed at: $Date: 2002/05/19 17:20:49 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron.netinputs;

import java.util.*;
import mosaic.sim.neuron.*;

/** This class implements a spike source using rate coding.
 *  It does for a vector what RateCodedInputSingle does for a single real
 *  numbered value. In fact, it uses RateCodedInputSingle for doing the real
 *  work when given a input vector.
 *  This class implements VectorInput, as does TemporalCodedInput. Therefore,
 *  these coding schemes should be easily interchangeable.
 */
public class RateCodedInputVector implements VectorInputCoder {
	/** The list of outputs that used for emitting spike events. The number of
	 *  outputs is equal to the length of the input vector.
	 */
	protected ArrayList outputs = null;

	/** Creates a new TemporalCodedInput object and initializes the outputs.
	 *  @param num The number of outputs, it must be equal to the length of the
	 *             analog input vectors that will be given to enterVector().
	 */
	public RateCodedInputVector(int num) {
		outputs = new ArrayList(num);
		for (int i=0; i<num; i++)
			outputs.add(i, new RateCodedInputSingle());
	}

	/** Implementation of VectorInput.enterVector. */
	public void enterVector(double[] vector) throws CodingException {
		if (vector.length != outputs.size())
			throw new CodingException("Vector length does not match the number of outputs.");
		for (int i=0; i<outputs.size(); i++)
			((RateCodedInputSingle) outputs.get(i)).setRate(vector[i]);
	}

	/** Implementation of VectorInput.getOutput. */
	public NeuronOutput getOutput(int index) {
		if (index >= 0 && index < outputs.size())
			return (NeuronOutput) outputs.get(index);
		else
			throw new ArrayIndexOutOfBoundsException();
	}
}
