/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: VectorInputCoder.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.1 $
 *  Last changed at: $Date: 2002/04/14 14:42:06 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.netinputs;

import mosaic.sim.neuron.*;

/** This is an interface for giving a one-dimensional vector of real numbered
 *  values to the network. It must be implemented by a class with a defined
 *  Spike coding scheme (the so-called coder).
 *
 *  @see MatrixInputCoder
 */
public interface VectorInputCoder {
	/** Enters a new real numbered input vector into the simulation.
	 *  @param vector The input vector to be used.
	 */
	public void enterVector(double[] vector) throws CodingException;

	/** Returns the output with the given index.
	 *  @param index The number of the output to return.
	 */
	public NeuronOutput getOutput(int index);
}
