/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: VectorOutputListener.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.1 $
 *  Last changed at: $Date: 2002/04/14 15:16:32 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.netoutputs;

/** This interface must be implemented by objects that want to receive the
 *  real numbered, one-dimensional vectors computed by the network. The object
 *  receives an event whenever an output cycle is complete and the real
 *  numbered output values have been computed.
 *  Note that these output events are generated by some Spike decoder.
 *
 *  @see MatrixOutputListener
 */
public interface VectorOutputListener {
	/** Receive the system output as a real numbered vector.
	 *  @param vector The real numbered values corresponding to the spike series
	 *                using the respective coding scheme implemented by the
	 *                Spike decoder.
	 */
	public void outputEvent(double[] vector);
}