
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class HashtableOfVectors          -------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------



package mosaic.util;

import java.util.*;
import java.lang.*;

/**
* Class HashtableOfVectors implements a Hashtable which contains Vectors as values
* which contain the elements.
* Use this class to realize a mapping of keys to a set (!!) of elements.
* Remark: Use putElem instead of put, because overwriting put
*         and calling super.put of Hashtable does not work.
*/
public class HashtableOfVectors extends Hashtable {

  public HashtableOfVectors() {
  }

  public HashtableOfVectors(int i) {
      super(i);
  }

  public HashtableOfVectors(int i, float f) {
      super(i, f);
  }

  /** Gets the Enumeration of all values in the VectorHashtable. */
  public   Enumeration elements () {
      Vector allElements = new Vector();
      Enumeration keys = keys();
      while (keys.hasMoreElements())  {
          Enumeration elems = elements(keys.nextElement());
          while (elems.hasMoreElements()) {
              allElements.addElement(elems.nextElement());
          }
      }
      return allElements.elements();
  }

  /** Gets the Enumeration of values for that key. */
  public   Enumeration elements (Object key) {
      Vector v = getVector(key);
      if (v == null) {
          v = new Vector();
      }
      return v.elements();
  }

  /** Gets the vector containing the elements for that key.*/
  public   Vector getVector (Object key) {
      Object val = get(key);
      if (val != null) {
          Vector v;
          try {
            v = (Vector)val;
          }
          catch (ClassCastException e) {
            v = null;
          }
          return v;
      }
      else
          return null;
  }

  /** Tests if the value is contained in any vector for any key. */
  public   boolean contains (Object value) {
      Enumeration keys = keys();
      boolean notFound = true;
      while (keys.hasMoreElements() && notFound)  {
          if (contains(keys.nextElement(), value)) {
              notFound = false;
              return true;
            }
        }
      return false;
  }

  /** Tests if the value is contained in any vector for any key. */
  public   boolean contains (Object key, Object value) {
      Vector v = getVector(key);
      if (v != null)
          return v.contains(value);
      else
          return false;
  }

// Java bug documentation: -----------------------------------------------------------------------------------
// Overwriting put and calling super.put does not work.
// From time to time calls the put method twice before calling put of Hashtable.
// -----------------------------------------------------------------------------------------------------------
   /** Adds a value to the vector of values for that key. */
    public   Object putElem(Object key, Object value) {
        Vector v = getVector(key);
        if (v == null) {
          v = new Vector();
            put(key, v);
        }
        v.addElement(value);
        return value;
    }

    /** Removes  a value from the vector of values for that key. */
    public   Object remove (Object key, Object value) {
        Vector v = getVector(key);
        if (v != null) {
            v.removeElement(value);
        }
        // delete vector when empty
        if (v.isEmpty()) {
          // hopefully this works
          remove(key);
        }
        return value;
    }

    /** Removes all elements from the vector for that key. */
    public   void removeAll (Object key) {
      Vector v = getVector(key);
        if (v != null) {
            v.removeAllElements();
        }
        // delete vector because empty
       remove(key);
    }

}
