package mosaic.util.custom;

import java.awt.Component;
import java.awt.Label;
import java.beans.*;
import java.lang.reflect.Array;

/**
 * Makes Properties of Class CustomVector visually editable
 * with the CustomArrayPropEditor.
 *
 * @version 17 Jan 1999
 * @author Thomas Hilpold
 */
public class CustomVectorEditor extends PropertyEditorSupport
        implements CustomArrayEditorAdapter {

    public CustomVectorEditor() {
        super();
        if (DEBUG[0]) OutFrame.out(this.toString() + " CREATED");
    }

    public String getAsText () {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETASTEXT : "
            + super.getValue());
        return super.getAsText();
    }

    public Object getValue() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETVALUE : "
            + super.getValue());
        return super.getValue();
    }

    public void setValue(Object value) {
        if (DEBUG[0]) OutFrame.out(this.toString() + " SETVALUE : " + value);
        CustomVector curVal = (CustomVector)super.getValue();
        if (    ((value != null) && (value.equals(curVal)))
             || ((value == null) && (curVal == null))
           ) {
            if (DEBUG[0]) OutFrame.out(this.toString() + " NOUPDATE : " + value);
            //no update needed
        }
        else {
            if (DEBUG[0]) OutFrame.out(this.toString() + " UPDATE : " + value);
            array = vectorToArray((CustomVector) value);
            super.setValue(value); //will also fire
        }
    }

    /**
     * not supported
     */
    public void setAsText(String text)
            throws java.lang.IllegalArgumentException {
        OutFrame.out(this.toString() + " SETASTEXT FAIL!" + text);
        throw new java.lang.IllegalArgumentException();
    }

    /**
     * not supported
     */
    public String[] getTags() {
        return null;
    }

    /**
     * Syntax:
     * new Integer[]{new Integer(3) , ...};
     */
    public String getJavaInitializationString() {
      CustomVector cur;
      cur = (CustomVector) super.getValue();
      OutFrame.out(this.toString() + " GETINIT : " + cur);
      return ("new sim.custom.CustomVector(" +
            new InitStringCreator().getInitStringForArray(array) + ")");
            //",\r\n "+ cur.getComponentType().getName() +".class)");
    }

    /** @return true */
    public boolean supportsCustomEditor() {
        return true;
    }

    /** creates and returns a CustomArrayPropEditor */
    public java.awt.Component getCustomEditor() {
        if (DEBUG[0]) OutFrame.out(this.toString() + "  getCustomEd with: " + super.getValue());
        if (super.getValue() != null) {
            Component ret;
            try {
                ret = new CustomArrayPropEditor(this);
                OutFrame.out("" + this +" CUSTOMED: " + ret.toString());
            } catch (Exception e) {
                OutFrame.out(this.toString() + "  :" + e.getMessage());
                ret = new Label("" + this +" NULL - CUSTOMED");
            };
            return ret;
        }
        else {
            if (DEBUG[0]) OutFrame.out(this.toString() +"VALUE NULL & CUSTOMEDIT!");
            return new Label("VALUE WAS NULL - CANNOT EDIT!");
        }
    }

    //
    // PropertyChangeSupport Methods
    // (no changed behaviour but debugging output)
    //

    public void addPropertyChangeListener(PropertyChangeListener listener) {
      if (DEBUG[0]) OutFrame.out(this.toString() + "  Add PropertyChangeListener "+listener);
      super.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
      if (DEBUG[0]) OutFrame.out(this.toString() + "  Removed PropertyChangeListener "+listener);
      super.removePropertyChangeListener(listener);
    }

    public void fire() {
      if (DEBUG[0]) OutFrame.out(this + "fire ! ");
      super.firePropertyChange();
    }

    //
    // Implementing Interface CustomArrayEditorAdapter
    //

    public void setAsArray(Object arr) {
        if (DEBUG[0]) OutFrame.out(this.toString() + " setAsArray :" + arr);
        //assert arr instanceof Object[]
        if (!arr.equals(array)) {
            array = (Object[]) arr;
            super.setValue(arrayToVector(array));
        }
    }

    public Object getAsArray() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " getAsArray: " + array);
        return array;
    }


    //
    // Conversion Methods for Array <-> Vector
    //

    private CustomVector arrayToVector(Object[] arr) {
        CustomVector ret;
        if (DEBUG[0]) OutFrame.out(this.toString() + " arrayToVector " + arr);
        ret = new CustomVector(arr.length, arr.getClass().getComponentType());
        //assert array not null and Array
        for (int i = 0; i < arr.length; i++)
            ret.addElement(arr[i]);
        return ret;
   }

    private Object[] vectorToArray(CustomVector cus) {
        Object[] ret;
        if (DEBUG[0]) OutFrame.out(this.toString() + " vectorToArray "+ cus);
        ret = (Object[]) Array.newInstance(cus.getComponentType(), cus.size());
        for (int i = 0; i < ret.length; i++) {
            if (DEBUG[0]) OutFrame.out(this.toString() + "obj "+ i +" "+ cus.elementAt(i));
             ret[i] = cus.elementAt(i);
        }
        if (DEBUG[0]) OutFrame.out(this.toString() + "finished ");
        return ret;
    }

    //
    // Data
    //

    /** The Object[] that can be edited by CustomArrayPropeditor */
    private Object[] array;

    //
    // Controlling output for debugging purposes
    //

    private boolean[] DEBUG = new boolean[] {false};
}
