package mosaic.util.custom;

import java.beans.PropertyEditorSupport;

/**
 * PropertyEditor which contains a visual Editor (PropertySheet)
 * for Properties that are Beans.
 * <p>
 * Ensure that an appropriate constructor exists in the Beans class
 * and the property is never set to null. Set the Properties, which should be
 * visible in the PropertySheet using a BeanInfo Class.
 * <p>
 * The InitString contains a Constructor call with
 * constructor calls to all properties of the Bean sorted alphabetically
 * in increasing order  with the Name of the Property as criterium.
 * <p>
 * Bean will be represented as String with bean.toString().
 * <p>
 * A CustomEditor can be opened, that supports a PropertySheet.
 * <p>
 *
 * @author Thomas Hilpold
 * @version 10 Feb 1999
 */
public class GenericBeanEditor extends PropertyEditorSupport
  implements CustomTagsPropertyEditor {

    /** array with switches to control runtime debug output*/
    protected boolean[] DEBUG = new boolean[] {false, false};

    /** non - argument constructor */
    public GenericBeanEditor() {
        super();
        if (DEBUG[0]) OutFrame.out(this.toString() + " NEWINSTANCE ");
    }

    /** returns only the ClassName*/
    public String getAsText () {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETASTEXT ");
        return super.getAsText();
    }

    /** also returns null */
    public Object getValue() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETVALUE: " + super.getValue());
        return super.getValue();
    }

    /** sets anything, even null */
    public void setValue(Object value) {
        if (DEBUG[0]) OutFrame.out(this.toString() + " SETVALUE: " + value);
        if (value != super.getValue()) {
            super.setValue(value);
        }
    }

    /** not supported */
    public void setAsText(String text)
            throws java.lang.IllegalArgumentException {
        if (DEBUG[0]) OutFrame.out(this.toString() + " SETASTEXT: " + text);
        throw (new IllegalArgumentException("Not Supported!"));
    }

    /**implements interface CustomPropertyEditor*/
    public boolean supportsCustomTags() {
        if (DEBUG[1]) OutFrame.out(this.toString() + " SUPPORTSCUSTOMTAGS ");
        return false;
    }

    /**implements interface CustomPropertyEditor*/
    public String[] getCustomTags() {
        if (DEBUG[1]) OutFrame.out(this.toString() + " GETCUSTOMTAGS ");
        return null;
    }

    /** uses an Initstringcreator which uses the only the value - object */
    public String getJavaInitializationString() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETJAVAINITSTRING ");
        InitStringCreator aISC = new InitStringCreator();
        return aISC.getInitStringForBean(getValue());
    }

    /** declares that there is a custom editor available */
    public boolean supportsCustomEditor() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " SUPPORTSCUSTOMEDITOR ");
        return true;
    }

    /** returns a visual Customeditor with a Propertysheet */
    public java.awt.Component getCustomEditor() {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETCUSTOMEDITOR ");
        return (new PropertyCustomEditor(this));
    }
}
