package mosaic.util.custom;

import java.beans.*;
import java.lang.reflect.Method;
import java.lang.reflect.Array;

/**
 * Instances are capable of creating javaInitStrings of a given bean or
 * Array of any Objects for which a PropertyEditor can be found.
 * <p>
 * For usage see GenericBeanEdior and ArrayEditor.
 *
 * @version 17 Feb 1999
 * @author Thomas Hilpold
 * @see GenericBeanEditor
 * @see ArrayEditor
 */
public class InitStringCreator {

  // also works for primitive type arrays
  public String getInitStringForArray(Object anArray) {
      String ret;
      array = anArray;
      OutFrame.out("getInitb - array - called !");
      if (anArray != null) {
          analyzeArray();
          ret = "new "+ array.getClass().getComponentType().getName()+ "[] {";
          ret += getJavaActualParameterString();
          ret += "}";
          return ret;
      }
      return "null";
  }

  public String getInitStringForBean(Object aBean){

  // it should be possible to have options when creating the InitString which
  // consists of constructor calls
  // perhaps expert properties, etc. might help
    String ret;
    bean = aBean;
    if (bean != null) {
       analyzeBean();
       ret = "new "+ aBean.getClass().getName()+ "(";
       ret += getJavaActualParameterString();
       ret += ")";
       OutFrame.out("" + this + "getInit returned: " + ret);
       return ret;
    }
    else {
        OutFrame.out(""+ this + "Bean was null");
        return "null";
    }
  }

  private String getJavaActualParameterString(){

        //gets All initstrings of PEs or getAsTExt
        String ret = new String("\r\n            ");
        for (int i =0; i < numOfProps; i++) {
            if (propEditors[i] != null) {
                try {
                    if (propEditors[i].getJavaInitializationString() != null) {
                        ret = ret + propEditors[i].getJavaInitializationString();
                    }
                    else if (propEditors[i].getAsText() != null) {
                        ret = ret + propEditors[i].getAsText();
                    }
                    else if (propEditors[i].getValue() != null) {
                        ret = ret + propEditors[i].getValue().toString();
                    }
                    else System.out.println("Error in getJavaActualParameterString");
                } catch (Exception e) {
                    e.printStackTrace();
                };
            }
            else {
                System.out.println("No propEditor for Property number: "+i);
                ret += "null";
            }
            if (i < (numOfProps - 1))ret += ", \r\n            ";
            //if (bean.getClass().isArray()) ret += "\r\n";
        }
        return ret;
  }
      private void analyzeBean(){
       Object curValue;
       Object[] args =new Object[0];
       PropertyDescriptor[] propDescs;
       if (bean != null) {
         try {
             propDescs = Introspector.getBeanInfo(bean.getClass()).getPropertyDescriptors();
          } catch (Exception e) {
              propDescs = null;
              System.out.println("No PropertyDescriptors");
          };
          if (propDescs != null) {
              PropertyDescSorter.sortAlpha(propDescs);
              numOfProps = propDescs.length;

              //initialize Arrays except propDescs
              propEditors = new PropertyEditor[numOfProps];
              for (int i = 0; i < numOfProps; i++) {

                  //ask propDesc if explicit Editor used
                  if (propDescs[i].getPropertyEditorClass() != null) {
                      try{
                        propEditors[i] =
                          ((PropertyEditor)propDescs[i].getPropertyEditorClass().newInstance());
                      }catch(Exception e) {
                          System.out.println(e.toString());
                          propEditors[i] = null;
                      }
                  }
                  else {
                    Class propType;
                    propType = propDescs[i].getPropertyType();
                    if (propType != null)
                            propEditors[i] =
                              PropertyEditorManager.findEditor(propType);
                  }
                      try {
                          curValue = propDescs[i].getReadMethod().invoke(bean,args);
                      }catch (Exception e) { curValue = null; /* !readAble*/ };
                      // some editors dont like null to be set, so we only set it
                      // if necessary
                      if ((propEditors[i] != null)
                          && (curValue != propEditors[i].getValue())) {
                          try {
                              propEditors[i].setValue(curValue);
                          } catch (Exception e) {
                            System.err.println("" + this +
                                               "Error while sending setValue("+curValue+
                                               ") to " + propEditors[i]);
                            };
                      }
              }
          }
       }
    }

    private void analyzeArray(){
        numOfProps = Array.getLength(array);

        //initialize Arrays except propDescs
        propEditors = new PropertyEditor[numOfProps];
        for (int i = 0; i < numOfProps; i++) {

            //ask propDesc if explicit Editor used
            if (Array.get(array,i) != null) {
                Class test = Array.get(array, i).getClass();

                // if we have primitives, we use the correct type
                // Array.get always returns an Object (wraps if necessary)
                if (array.getClass().getComponentType().isPrimitive())
                    test = array.getClass().getComponentType();
                System.out.println(test.toString());
                propEditors[i] = PropertyEditorManager.findEditor(test);

                /* if there is no PE for class of component,
                 * try to find an Editor for componenttype which will be a superclass
                 * a PE for Distribution for example will accept any special Distribution
                 */
                if (propEditors[i] == null) {
                    test = array.getClass().getComponentType();
                    propEditors[i] = PropertyEditorManager.findEditor(test);
                }

                // some editors dont like null to be set, so we only set it
                // if necessary
                Object val = Array.get(array, i);
                if ((propEditors[i] != null) && (val != propEditors[i].getValue())) {
                    try {
                        propEditors[i].setValue(val);
                    } catch (Exception e) {
                        System.err.println(""+ this +
                                           "Error while sending setValue(" + val +
                                           ") to " + propEditors[i]);
                    };
                }

            }
        }
    }

    //
    // Data
    //

    private Object bean;
    private Object array;
    private PropertyEditor[] propEditors;
    private int numOfProps;
}

