package mosaic.util.img;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import javax.swing.ImageIcon;

/**
 * This class is responsible for loading resources such as Images and ImageIcons
 * relative to the position of this class.
 *
 * The static methods are thread-safe.
 *
 * An unknown.gif should be available - It will be loaded if a given string
 *
 *
 * @Author Thomas Hilpold
 */
public class Img {

    static URL unknown;
    static Integer STATIC_LOCK;

    static {
        STATIC_LOCK = new Integer(-1);
        unknown = getSimResource("unknown.gif");
    }

    /** creates an URL relative to this class */
    static public URL getSimResource(String resString) {
        URL u;

        synchronized (STATIC_LOCK) {
            u = Img.class.getResource(resString);
            if (u != null) {
                return u;
            } else {
                System.err.println("Img.class: Could not load Resource: "
                        + resString);
                return null;
            }
        }
    }

    /** creates an ImageIcon with an image loaded from this packages location*/
    static public ImageIcon createSimImageIcon(String resString) {
        URL u;

        synchronized (STATIC_LOCK) {
            u = getSimResource(resString);
            if (u == null) {
                u = unknown;
            }
            return new ImageIcon(u);
        }
    }

    /** creates an Image loaded from this packages location */
    static public Image createSimImage(String resString) {
        URL u;

        synchronized (STATIC_LOCK) {
            u = getSimResource(resString);
            if (u == null) {
                u = unknown;
            }
            return Toolkit.getDefaultToolkit().createImage(u);
        }
    }
}
