
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------
// class VariableView    -------------------------------------------------------
// -----------------------------------------------------------------------------

package mosaic.vis.general;

import java.awt.*;
import java.util.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;
import javax.swing.JComponent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentAdapter;

/**
* Base class for the output view to visualize variable values.
* Extends BasicVariableObserver as a base implementation of VariableChangeListener.
*
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @author Bernhard Reitinger --> optimized 08/02/2000
* @version %I% %G%
*/
public class VariableView extends BasicVariableObserver implements ComponentListener{

    protected java.awt.Graphics cachedG = null;
    protected boolean resized = false;

    private Integer LOCK = new Integer(-1);
    private Integer LOCK1 = new Integer(-1);
//     private ComponentAdapter compAdapter;
    protected Rectangle clipRect = null;

    private boolean needGraphicsContext = true;
    private boolean registeredAtParents = false;

    /** Default constructor ValueView */
    public VariableView() {
	    setLabel (" ");
	    setBackground(new Color(140, 200, 255));
	    setForeground(Color.black);
	    setDoubleBuffered(false);
//	    setOpaque(true);
        addComponentListener( this);
    }


    // variableChange ----------------------------------------------------------

    /**
     * Reacts to a variableChange event by setting the value and repainting the view.
     */
    public void variableChange(VariableChangeEvent e) {

        //System.out.println("VariableView::variableChange");
        super.variableChange(e);
    	paintCached();
        if (!registeredAtParents) {
            // find next parent who is instance of JPanel...
            registeredAtParents = true;
            Container parent = getParent();
            while (parent != null) {
                if (parent instanceof Container) { // parent found, that is JComponent
                    parent.addComponentListener(this);
                }
                parent = parent.getParent(); // get next Parentcomponent
            }
        }

    }

    public boolean isOptimizedDrawingEnabled() { return false;}


    // paint -------------------------------------------------------------------

    /**
     * Paints the view;
     * by calling <code>super.paint</code> which paints the background and
     * calling polymorphic method <code>paintValue</code> to delegate painting
     * of the values to specialized views.
     *
     * @param g the graphics context of the view
     */
    public void paintWithGfx(Graphics g) {
        if (isShowing()) {
            int width = getSize().width;
            int height = getSize().height;
            g.setColor(getBackground());
            g.fill3DRect(0, 0, width, height, true);
            g.setColor(getForeground());
            g.setFont(getFont());
            FontMetrics fm = g.getFontMetrics();
            g.drawString(getLabel(), (width - fm.stringWidth(getLabel())) / 2, height - fm.getMaxDescent() -2);
            paintValue (g);
        }
    }

    public void paintComponent(Graphics g) {
// 	System.out.println("variableview::paintcomponent");

	synchronized(LOCK) {
	    paintWithGfx(g);
	}
    }

    public void paintCached() {
        // 	System.out.println("paintcached");
        if (cachedG == null)
            cachedG = getGraphics();

        synchronized(LOCK1) {
            if (needGraphicsContext) {
                cachedG = getGraphics();
                needGraphicsContext = false;
            }
        }

        if (cachedG != null) {

            synchronized(LOCK) {
            // setting the clipping region
            if (clipRect != null) {
                cachedG.setClip(clipRect.x, clipRect.y, clipRect.width, clipRect.height);
            }
            paintWithGfx(cachedG);
        }
    }

// 	if (getParentModelComponent() instanceof JComponent)
// 	    ((JComponent)getParentModelComponent()).repaint();

    }

    /** Empty method paintValue */
    public void paintValue(Graphics g) { }


    // size --------------------------------------------------------------------

    /** Gets minimum size for the LayoutManager */
    public Dimension getMinimumSize() {
	return new Dimension(10,10);
    }

    /** Gets preferred size for the LayoutManager */
    public Dimension getPreferredSize() {
	return new Dimension(40,24);
    }

    public boolean isResized() {
	    return resized;
    }

    public void setResized(boolean b) {
    	resized = b;
    }


    // implement interface Component Listener ----------------------------------
    public void componentHidden(ComponentEvent e) {
        synchronized(LOCK1) {
            needGraphicsContext = true;
        }
        //System.out.println("VariableView ComponentEvent hidden ("+e.getComponent().getClass().getName()+")");

    }

    public void componentMoved(ComponentEvent e) {
        synchronized(LOCK1) {
            needGraphicsContext = true;
        }
        //System.out.println("VariableView ComponentEvent moved ("+e.getComponent().getClass().getName()+")");

    }

    public void componentResized(ComponentEvent e) {
        synchronized(LOCK1) {
            needGraphicsContext = true;
        }
        resized = true;
        //System.out.println("VariableView ComponentEvent resized ("+e.getComponent().getClass().getName()+")");

    }

    public void componentShown(ComponentEvent e) {
        synchronized(LOCK1) {
            needGraphicsContext = true;
        }
        //System.out.println("VariableView ComponentEvent shown ("+e.getComponent().getClass().getName()+")");

    }


// animated ----------------------------------------------------------------------------------------

    boolean animated = false;

    public void setPerformAnimation(boolean animate) {
        animated = animate;
    }

    public boolean getPerformAnimation() {
        return animated;
    }



    // overwrite addTo method ------------------------------------------------------------------------------------

/*    public void addTo(ModelComponent comp) {
    	super.addTo(comp);
     	if (comp instanceof JComponent) {
	      ((JComponent)comp).addComponentListener(this);
     	}
    }

    public void removeFrom (ModelComponent comp) {
      if (comp != null) {
      	if (comp instanceof JComponent) {
  	      ((JComponent)comp).removeComponentListener(this);
        }
      }
    }

  */
  // double buffering ------------------------------------------------------------------------------------------

//     /** Graphics object for off screen  painting */
//     protected Graphics bufferG;
//     /** image for off screen painting */
//     Image offScreenBuffer;

//     /** Updates the component by first painting off screen and then copying the
//      * image into the real graphics.
//      */
//     public void update(Graphics g) {
//         Graphics gr;
//         // sure we keep our offscreen buffer is the same size as the container
//         if (offScreenBuffer==null ||
// 	    (! (offScreenBuffer.getWidth(this) == this.getSize().width
// 		&& offScreenBuffer.getHeight(this) == this.getSize().height))) {
// 	    offScreenBuffer = this.createImage(getSize().width, getSize().height);
//         }
//         gr = offScreenBuffer.getGraphics();
//         paint(gr);
//         g.drawImage(offScreenBuffer, 0, 0, this);
//     }



}

