//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class Switch ----------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.input;

import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.event.*;

/**
* Visualizes a rounded switch with labels for on and off state.
* is updated if connected to a variable as variable change listener and
* allows user to switch between the two states, by clicking on the drawing.
* <p>
* Use the view by adding it as VariableChangeListener to a variable of arbitrary type .
* Use the input functionality by registering a userInputListener at this.
* <p>
* Costumization:
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set the on-label by <code>setOnLabel</code>
* <li> set the off-label by <code>setOffLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*/


public class Switch extends BooleanInputView {

  public Switch() {
    super();
    try  {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.addMouseListener(new MouseAdapter() {

      public void mousePressed(MouseEvent e) {
        switchState(e);
      }
    });
  }
// on/off labels ---------------------------------------------------------------

    /** variable to store the text displayed when true */
  private String onLabel= "ON";
  /** Gets the text to be displayed when true */
  public String getOnLabel() { return onLabel; }
  /** Sets the text to be displayed when true */
  public void setOnLabel(String htext) { onLabel= htext; }

  /** variable to store the text displayed when false */
  private String offLabel= "OFF";
  /** Gets the text to be displayed when false */
  public String getOffLabel() { return offLabel; }
  /** Sets the text to be displayed when false */
  public void setOffLabel(String htext) { offLabel= htext; }

// Hightlightcolor -------------------------------------------------------------

  /** variable to store the hightlightingcolor */
  private Color highlightColor= Color.yellow;
  /** Gets the hightlightcolor */
  public Color getHighlightColor() { return highlightColor; }
  /** Sets the hightlightcolor */
  public void setHighlightColor(Color hcolor) { highlightColor = hcolor; }

// paintValue ------------------------------------------------------------------

    /**
    * Paints the boolean value.
    * @param g the graphics context of the view
    */
    public void paintValue(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    // Draw circle
    g.setColor(getForeground());
    if (getBoolValue() != null)
    { if (!getBoolValue().booleanValue()) {
      // OFF
        g.fillPolygon(new int[]{width/5, 3*width/20, 2*width/5},
                      new int[]{height- (7*width/10), height-width, height-(8*width/10)},3);
        g.setColor(Color.black);
        g.drawPolygon(new int[]{width/5, 3*width/20, 2*width/5},
                      new int[]{height- (7*width/10), height-width, height-(8*width/10)},3);

        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getOnLabel(), width - fm.stringWidth(getOnLabel()),
                     fm.getMaxAscent() );
        g.setColor(getHighlightColor());
        g.drawString(getOffLabel(), 0, fm.getMaxAscent());
      }
      else {
      // ON
        g.fillPolygon(new int[]{4*width/5, 8*width/10, 3*width/5},
                      new int[]{height- (7*width/10), height-width, height-(8*width/10)},3);
        g.setColor(Color.black);
        g.drawPolygon(new int[]{4*width/5, 8*width/10, 3*width/5},
                      new int[]{height- (7*width/10), height-width, height-(8*width/10)},3);

        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getOffLabel(), 0, fm.getMaxAscent());
        g.setColor(getHighlightColor());
        g.drawString(getOnLabel(), width - fm.stringWidth(getOnLabel()),
                     fm.getMaxAscent() );

      }
    }
    else { // if no value accessible, draw if it was ON
      g.fillPolygon(new int[]{width/5, 3*width/20, 2*width/5},
                    new int[]{height- (7*width/10), height-width, height-(8*width/10)},
                    3);
      g.setColor(Color.black);
      g.drawPolygon(new int[]{width/5, 3*width/20, 2*width/5},
                    new int[]{height- (7*width/10), height-width, height-(8*width/10)},
                    3);
      g.setFont(getFont());
      FontMetrics fm = g.getFontMetrics();
      g.drawString(getOnLabel(), width - fm.stringWidth(getOnLabel()),
                   fm.getMaxAscent() );
      g.setColor(getHighlightColor());
      g.drawString(getOffLabel(), 0, fm.getMaxAscent());
    }
    g.setColor(getForeground());
    g.fillOval(width/10,height-(9*width/10),4*width/5, 4*width/5);
    g.setColor(Color.black);
    g.drawOval(width/10,height-(9*width/10),4*width/5, 4*width/5);

  }

// mouse events ----------------------------------------------------------------

  /**
  * gets called if user presses mouse over this, toggle boolean value
  */
  void switchState(MouseEvent e) {
//    if (! isHelpMode()) {
        setValue( new Boolean(!(getBoolValue().booleanValue())) );
        triggerEvent();
        repaint();
//    }
  }

    public static void main(String[] args) {
	javax.swing.JFrame f = new javax.swing.JFrame();
	Switch s = new Switch();
	javax.swing.JPanel panel = new javax.swing.JPanel();

	panel.setLayout(null);
	s.setLocation(0,0);
	s.setBounds(0,0,200,100);
	panel.add(s);
	f.getContentPane().add(s);
	f.setSize(500,100);
	f.setLocation(100,100);
	f.show();
	
    }

}


