package mosaic.vis.output;

import java.awt.*;
import javax.swing.*;
import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.control.AnimatedView;
import mosaic.vis.general.VariableView;


abstract public class AnimatedImageView extends VariableView implements AnimatedView {

    public AnimatedImageView() {
        this(null);
    }

    /**
     * Constuctor with initialization of images.
     */
    public AnimatedImageView(ImageIcon[] images) {
        setImages(images);
    }

    // ---------------------------------------------------------------------------------------------

    protected  boolean isOn;
    private  double lastPaintTime;

    /** will set the spokes to inital position */
    public void reset() {
        lastPaintTime = 0.0;
        repaint();
    }

    // Property images -----------------------------------------------------------------------------
    protected ImageIcon[] images;

     /** Get the iamges */
    public ImageIcon[] getImages() {
        return images;
    }

    /** Set the images */
    public void setImages(ImageIcon[] images) {
        this.images = images;
        if (this.images != null) {
            setNrImages(this.images.length);
            setCurImageIndex(0);
        } else {
            setNrImages(0);
            setCurImageIndex(-1);
        }
    }


    // Property nrImages ---------------------------------------------------------------------------
    protected int nrImages = 0;

     /** Get the number of the images */
    protected int getNrImages() {
        return nrImages;
    }

    /** Set the number of the images */
    protected void setNrImages(int nr) {
        nrImages = nr;
    }

    // Property curImageIndex ----------------------------------------------------------------------
    protected int curImageIndex = 0;

     /** Get the index of the current image */
    protected int getCurImageIndex() {
        return curImageIndex;
    }

    /** Set the index of the current image */
    protected void setCurImageIndex(int idx) {
        curImageIndex = idx;
    }

    /** Increment the index of the current image */
    protected void incrCurImageIndex() {
        curImageIndex = (curImageIndex + 1) % nrImages;
    }

    // Property imagePaintTime ---------------------------------------------------------------------
    protected double imagePaintTime = 0.5;

     /** Get the time to paint one image */
    protected double getImagePaintTime() {
        return imagePaintTime;
    }

    /** Set the time to paint one image */
    protected void setImagePaintTime(double time) {
        imagePaintTime = time;
    }

    // Property cycleTime ---------------------------------------------------------------------
    protected double cycleTime = 1;

     /** Get the time to paint all images once (multiplied by the speed for numbers views) */
    public double getCycleTime() {
        return cycleTime;
    }

    /** Set the time to paint all images once (multiplied by the speed for numbers views) */
    public void setCycleTime(double time) {
        cycleTime = time;
    }


    // ---------------------------------------------------------------------------------------------

    /**
     * Will paint the images in a cycle
     */
    public boolean performAnimationStep(double time) {
        double deltaT = time-lastPaintTime;

        boolean doPaint = (isOn && (deltaT >= imagePaintTime) && getNrImages() > 0);
        if (doPaint) {
            paintCached();
            lastPaintTime = time;
            incrCurImageIndex();
        }
        return doPaint;
    }

    //boolean paintCallback = false;
    public void paint(Graphics g) {
        //paintBackgroundWithGfx(g);
        if (isOn) {
           paintWithGfx(g);
        }
    }

    final public void paintBackgroundWithGfx(Graphics g) {
        if (isShowing()) {
            Graphics2D g2 = (Graphics2D) g;
            // draw background
            g2.setColor(getBackground());
            g2.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
        }
    }

    final public void paintWithGfx(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        if (isShowing() && getNrImages() > 0) {
            g2.drawImage(images[curImageIndex].getImage(), 0, 0, this);
        }
    }
}
