package mosaic.vis.output;

import javax.swing.*;
import mosaic.sim.kernel.output.*;
import java.awt.event.*;
import java.awt.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;

public class ExpandableShortTrend extends AbstractModelComponent implements expandable{


    public ExpandableShortTrend() {
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    // Views integrated into an expandable view :
    ShortTrend trend = new ShortTrend();
    ExpandSymbol expandi = new ExpandSymbol();

    private void jbInit() throws Exception {

        expandedSize = new Dimension(200,100);
        collapsedSize = new Dimension(11,7);
        trendBounds = new Rectangle(1,1,
                                    expandedSize.width - 10,
                                    expandedSize.height - 2);


        this.setSize(collapsedSize);
        trend.setBounds(trendBounds);
        expandi.setBounds( 1, 1, 5, 10 );

        this.setBackground(Color.white);

        trend.setBackground(Color.white);
        trend.setTimeSpan(15.0);

        expandi.setBackground(Color.white);
        expandi.setForeground(Color.blue);
        expandi.setOwner(this);

        this.add(expandi, null);
    }

    // Access functions --------------------------------------------------------
    public ShortTrend getTrend() { return trend; }

    // -------------------------------------------------------------------------
    // properties --------------------------------------------------------------
    // -------------------------------------------------------------------------

    // expanding ---------------------------------------------------------------
    boolean expanded = false;
    boolean collapsingOrExpanding = false;
    Dimension collapsedSize, expandedSize;
    Rectangle trendBounds;

    public boolean isExpanded() { return expanded; }

    public void collapse() {
        if (expanded) {
            collapsingOrExpanding = true;
            this.remove(trend);
            this.add(expandi);
            expandi.setBounds( 1, 1, expandi.getWidth(), expandi.getHeight() );
            this.setSize(collapsedSize);
            collapsingOrExpanding = false;

            expanded = false;
        }
    }

    public void expand() {
        if (!expanded) {
            collapsingOrExpanding = true;
            trend.setBounds(trendBounds);
            this.add(trend);
            this.add(expandi);
            expandi.setBounds( expandedSize.width - expandi.getWidth()-2,
                               1,
                               expandi.getWidth(),
                               expandi.getHeight() );
            this.setSize(expandedSize);
            collapsingOrExpanding = false;

            expanded = true;
        }
    }

    // property timespan forwarded to trendview --------------------------------

    public void setTimeSpan(double ts) { trend.setTimeSpan(ts); }

    public double getTimeSpan() { return trend.getTimeSpan(); }

    // property color for pen nr nr of Trend -----------------------------------

    public void setPenColor(int nr, Color c) { trend.getPen(nr).setColor(c); }

    public Color getPenColor(int nr) { return trend.getPen(nr).getColor(); }

    // properties defining range for pen number nr of trend --------------------

    public void setMax(int nr, double m) {
        trend.getPen(nr).setMax(m);
    }

    public double getMax(int nr) {
        return trend.getPen(nr).getMax();
    }

    public void setMin(int nr, double m) {
        trend.getPen(nr).setMin(m);
    }

    public double getMin(int nr) {
        return trend.getPen(nr).getMin();
    }

    // properties defining size of trendView -----------------------------------
    // expect TrendView to be always larger than numberView, -------------------
    // if not ignore settings --------------------------------------------------

    public Dimension getExpandedSize() {
        return expandedSize;
    }

    public void setExpandedSize(Dimension size) {
        expandedSize.setSize(size);
        trendBounds = new Rectangle(1, 2, expandedSize.width - 10,
                                    expandedSize.height - 2);

    }

    public void setBounds( int x, int y, int w, int h) {
        // collapsedSize
        if (!collapsingOrExpanding)
        {  collapsedSize.setSize(w,h);
           expandi.setBounds( 1 , 1,
                              expandi.getWidth(), expandi.getHeight() );
        }
        super.setBounds(x,y,w,h);
    }

    // accessFunctions to add and remove pens to trend -------------------------

    public void addPen(DoubleVariable v, Color c) {
        trend.addPen(v, c);
    }

    public void removePen(int nr) {
        trend.deletePen(nr);
    }

    // -------------------------------------------------------------------------
    // paintComponent ----------------------------------------------------------

    public void paintComponent(Graphics g) {
        g.setColor(getBackground());
        g.fillRect(0,0, getWidth(), getHeight());
        g.setColor(getForeground());
        g.drawRect(0,0,getWidth()-1, getHeight()-1);
    }

}
