
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class BooleanView        ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;

/**
* Animation view to visualize boolean values.
* If true, paints the rectangle in foreground color and displays onText (T).
* If false, paints the rectangle in background color and displays offText (F).
* <p>
* Use the view by adding it as VariableChangeListener to a boolean variables.
* <p>
* <b>Costumization:</b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set the text to display if true by  <code>setOnText</code>
* <li> set the text to display if false by  <code>setOffText</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/
public class ExtBooleanView extends BooleanView {

    /** Default constructor BooleanView */
  public ExtBooleanView() {
    super();
  }


  // property onColor
  private Color onColor = Color.red;
  // sets color for string if boolean value is true
  public void setOnColor(Color onColor) { this.onColor = onColor; }
  // gets color for string if boolean value is true
  public Color getOnColor() { return onColor ; }

  // property onColor
  private Color offColor = Color.blue;
  // sets color for string if boolean value is false
  public void setOffColor(Color offColor) { this.offColor = offColor; }
  // gets color for string if boolean value is false
  public Color getOffColor() { return offColor ; }

// paintValue ------------------------------------------------------------------------------------------------

    /**
    * Paints the boolean value.
    * @param g the graphics context of the view
    */
    public void paintValue(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    if (onOff) {
        g.setColor(getBackground());
        g.fillRect(0,0,width,height);
        g.setColor(getForeground());
        g.drawRect(0,0,width-1,height-1);
        g.setColor(getOnColor());
        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }
    else {
        g.setColor(getBackground());
        g.fillRect(0,0,width,height);
        g.setColor(getForeground());
        g.drawRect(0,0,width-1,height-1);
        g.setFont(getFont());
        g.setColor(getOffColor());
        FontMetrics fm = g.getFontMetrics();
        g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }

  }

  public void paintWithGfx(Graphics g) {
    if (isShowing()) {
      paintValue (g);
    }
  }

}
