
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class NumberView         ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;
import java.beans.*;

/**
* Animation view to visualize numeric values.
* Can serve as variable change listeners for numeric variables.
* Displays the number using drawString.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger 
* @author Alois Stritzinger
* @version %I% %G%
*/
public class NumberView extends TextView {

  /** Default constructor NumberView */
  public NumberView() { }

// number value ----------------------------------------------------------------------------------------------

  /** variable to store the number object */
  Number number;

  /** Sets the number object received by a variableChange */
  public void setNumber(Number newNumber) {
    number= newNumber;
  }
  /** Gets the number object */
  public Number getNumber() { return number; }

// setValue --------------------------------------------------------------------------------------------------

  /**
  * Sets the value object received by a variableChange.
  * Specialized from ValueView.
  * @param newVal   new value object received from variableChange.
  */
  public void setValue (Object newVal) {
        if (newVal instanceof Number) {
            super.setValue(newVal);
            setNumber ((Number)newVal);
        }
        else
            System.out.println(">> tried to send non Number value to " + toString());
     }

// paintValue ------------------------------------------------------------------------------------------------

   /**
   * Paints the number value;
   * by putting out the text representing the number in the view.
   * @param g     the graphics context of the view
   */
   public void paintValue(Graphics g) {
      int width = getSize().width;
      int height = getSize().height;
      g.setFont(getFont());
      g.setColor(getForeground());
      FontMetrics fm = g.getFontMetrics();
      g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
            (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }
}


