
/**
* Animation view to visualize numeric values.
* Can serve as variable change listeners for numeric variables.
* Displays the number using drawString.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

package mosaic.vis.output;
import java.awt.*;
import java.util.*;
import java.beans.*;


public class ScaledFormattedNumberView extends FormattedNumberView{

    /** Default constructor NumberView */
    public ScaledFormattedNumberView() {
    }

// setValue --------------------------------------------------------------------------------------------------

  /**
  * Sets the value object received by a variableChange.
  * Specialized from NumberView in that the stored number is multiplied with the scale factor
  * @param newVal   new value object received from variableChange.
  */
  public void setValue (Object newVal) {
        if (newVal instanceof Number) {
            Number newNumber = new Double (((Number)newVal).doubleValue()*scaleFactor);
            super.setValue(newNumber);
            setNumber (newNumber);
        }
        else
            System.out.println(">> tried to send non Number value to " + toString());
     }

    // Properties
    // scaleFactor ------------------------------------------------------------
    double scaleFactor = 1000.0;

    /**
    * Sets the new scalingFactor.
    */
    public void setScaleFactor(double s) {
        scaleFactor = s;
    }

    /**
    * Gets the actual scalingFactor
    */
    public double getScaleFactor() {
        return scaleFactor;
    }

}
