package mosaic.vis.output;
import mosaic.util.Queue;
import java.awt.*;
import javax.swing.*;
import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.continuous.*;
import java.util.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.control.*;

public class ShortTrend extends AbstractModelComponent{

    // Data structure holds Pen's data and color, and works as variable observer
    // on its simulation Variable
    public class Pen extends BasicVariableObserver{
        Boolean LOCK = new Boolean(true);
        Queue xValues = new Queue(10,10);
        Queue yValues = new Queue(10,10);
        ShortTrend trend;
        double max = 100.0;
        double min = 0.0;
        Color color = Color.red;

        public Pen(ShortTrend owner,
                   DoubleVariable variable,
                   Color c) {
            color = c;
            if (variable != null)
                variable.addVariableChangeListener(this);
            trend = owner;
        }

        // interface variableChangeListener ------------------------------------

        /** Reacts to a variableChange event by adding a new Point to pens data.*/
        synchronized public void variableChange(VariableChangeEvent e) {
            if (e.getValue() instanceof Number) {
                xValues.enqueue(new Double(e.getChangeTime()));
                yValues.enqueue(e.getValue());
                trend.repaint();
            }
        }



        // AccessFunctions

        /** Returns array containing all x (time) values transformed to fit
            in trendwindow */
        public int[] getXValues() {
            int size = getNrOfValues();
            double actTime = getTime();
            int[] x = new int[size];
            for (int i = 0; i < size; i++) {
                x[i] = (int)( trend.getWidth() - ((actTime - ((Double)(xValues.getElementAt(i))).doubleValue() )
                              * trend.getWidth() / trend.getTimeSpan()) );
            }

            return x;
        }

        /** Returns array containing all y (value) values transformed to fit
            in trendwindows height */
        synchronized public int[] getYValues() {
            int size = getNrOfValues();
            double actTime = getTime();
            int[] y = new int[size];
            for (int i = 0; i < size; i++) {
                y[i] = (int)( trend.getHeight() -
                              (((Double)(yValues.getElementAt(i))).doubleValue() - min )
                              * trend.getHeight() / (max-min) );
            }

            return y;
        }

        synchronized public int getNrOfValues() {
           return xValues.getQueueLength();
        }

        // Sets max and min for viewing values
        public void setMax(double m) { max = m; }

        public double getMax() { return max; }

        public void setMin(double m) { min = m; }

        public double getMin() { return min; }

        public void setColor(Color c) { color = c; }

        public Color getColor() { return color; }


        /** Deletes all Values stored in data older than time */
        public void deleteValuesBefore(double time) {
            // rely on "short-circuit" evaluation
            boolean done = false;
            while (!done) {
                if (xValues.getQueueLength() > 0){
                    if ( ((Double)(xValues.getLast())).doubleValue() < time) {
                        xValues.dequeue();
                        yValues.dequeue();
                    } else {
                      done = true;
                    }
                } else {
                     done = true;
                }
            }
        }
    }

    // -------------------------------------------------------------------------
    // members -----------------------------------------------------------------
    // -------------------------------------------------------------------------

    Vector pens = new Vector();

    // -------------------------------------------------------------------------
    // methods -----------------------------------------------------------------
    // -------------------------------------------------------------------------

    public ShortTrend() {
        super();
    }

    public void paintComponent(Graphics g) {
        // Draw background
        g.setColor(getBackground());
        g.fill3DRect(0, 0, getWidth()-1, getHeight()-1,true);
        /*g.setColor(getForeground());
        g.draw3DRect(0, 0, getWidth()-1, getHeight()-1, true);*/

        Enumeration allPens = pens.elements();
        Pen pen;
        while (allPens.hasMoreElements()) {
            pen = (Pen)allPens.nextElement();
            paintTrajectory(pen, g);
        } 

    }

    public void paintTrajectory( Pen pen, Graphics g) {
        g.setColor(pen.color);
        pen.deleteValuesBefore(getTime() - getTimeSpan() );
        g.drawPolyline(pen.getXValues(), pen.getYValues(), pen.getNrOfValues() );
    }

    // -------------------------------------------------------------------------
    // Pen management ----------------------------------------------------------
    // -------------------------------------------------------------------------

    /** adds a new pen to trend */
    public void addPen(DoubleVariable variable, Color color) {
        pens.add(new Pen(this, variable, color));
    }

    /** deletes pen number nr from trend */
    public void deletePen(int nr) {
        if ( (0 <= nr) && (nr < pens.size()) ) {
            pens.removeElementAt(nr);
        }
    }

    /** returns Pen number nr */
    public Pen getPen(int nr) {
        if ( (0 <= nr) && (nr < pens.size()) ) {
            return ( (Pen)pens.elementAt(nr));
        } else {
            return null;
        }
    }


    // -------------------------------------------------------------------------
    // properties --------------------------------------------------------------
    // -------------------------------------------------------------------------
    double timespan = 60;

    public void setTimeSpan(double ts) { timespan = ts; }

    public double getTimeSpan() { return timespan; }


}
