
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class TextView           ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;
import mosaic.vis.general.VariableView;

/**
* Animation view to visualize textual representations of arbitrary values.
* Gives out the value as its textual representation using <code>toString</code> 
* <p>
* Use the view by adding it as VariableChangeListener to a variable of arbitrary type .
* <p>
* Costumization: 
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* 
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class TextView extends VariableView {

  /** Default constructor for TextView */
  public TextView() {
      text = ""; 
  }

//  text output ----------------------------------------------------------------------------------------------

  /** variable to hold the textual representation of the variable value*/
   protected String text;

  /** Sets the textual representation of the variable value*/
  public void setText(String newText) {
    text= newText;
  }

  /** Gets the textual representation of the variable value*/
  public String getText() {
    return text;
  }

// setValue --------------------------------------------------------------------------------------------------

  /** 
  * Specialized method to set a new value; 
  * also sets the textual represenation using <code>toString</code>. 
  */
  public void setValue (Object newVal) {
      super.setValue(newVal);
      String htext = "";
      if (getValue() != null) 
          htext = htext + getValue().toString();
      setText ( htext );
    }

// paint -----------------------------------------------------------------------------------------------------

  /**
  * Paints the object;
  * by giving out its textual representation.
  *
  * @param g the graphics context of the view
  */
  public void paintValue(Graphics g) {
      int width = getSize().width;
      int height = getSize().height;
      g.setColor(getForeground());
      g.setFont(getFont());
      FontMetrics fm = g.getFontMetrics();
      g.drawString(getText(), (width - fm.stringWidth(getText())) / 2,
          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
  }
}

